#ifndef _PTR_H_
#define _PTR_H_
#include <pgw.h>
#include <stdio.h>

#ifndef SEEK_SET
#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2
#endif



#define FGCOLOR "default"
#define BGCOLOR "black"

/*#define SCL_FORM "\\x0.8 %s"*/
#define SCL_FORM "%-8lg"
#define SCL_DIST 0.03
#define LB_DIST  0.07


/* default values */
#define DFGMX   120
#define DFGMY   120
#define STLNUM   1000
#define STLLENGTH 10.0


/* macro for error level */
#define INFO  1
#define WARN  2
#define ERROR 3
#define FATAL 4
#define INNER 9

/* macro for error message */
#define E_BadValue       "Bad value."
#define E_FileError      "\"%s\": Can't open file."
#define E_NoData         "No current data."
#define E_BadDataType    "Bad data type."
#define E_NoSuchGraph    "No such graph."
#define E_NoSuchWindow   "No such window."
#define E_OpenFile       "Open file before."
#define E_NoEnoughData   "No enough data. Can't draw graph."
#define E_NoEnoughLeng   "No enough length of vector."
#define E_NoEnoughDim    "No enough dimension."
#define E_CompNumReq     "Component number must be specified."
#define E_Unsuported     "\"%s\": Unsuported."


/* macro for data structure */
#define GRID 1
#define SEQ  2



/* macro for axis */
#define AX_NL 0
#define AX_X1 1
#define AX_X2 2
#define AX_X3 3
#define AX_V  4


/* macro for varius graph types */

/* PTRGraph types */
#define POINTS    1
#define LINE      2
#define STRING    4
#define FIXSTRING 5
#define ARROWS   11
#define GRAPHSET 51
#define LABEL    81
#define SCALE    91
#define SCLFRAME 92


/* macro for stream line mode */
#define EULER    1
#define RUNGE    2
#define FORWARD  1
#define BACKWARD 2
#define BOTH     3 



/* type definitions */

typedef struct {
    int type;
    int dim;
    int nx1;
    int nx2;
    int nx3;
    int nv;
    double* x1;
    double* x2;
    double* x3;
    double** v;
} PTRData;

typedef struct {
    double x;
    char lb[64];
} ScaleLabel;

typedef struct _PTRGraphList {
    union _PTRGraph* gp;
    struct _PTRGraphList* next;
} PTRGraphList;

typedef struct {
    int type;
    int vis;
    PGObjectList* pgl;
} PTRAnyGraph;

typedef struct {
    int type;
    int vis;
    PGObjectList* pgl;
    PTRData* dp;
    double* x1;
    double* x2;
    double* x3;
    PGP pgp;
} PTRPoints;
typedef struct {
    int type;
    int vis;
    PGObjectList* pgl;
    PTRData* dp;
    double* x1;
    double* x2;
    double* x3;
    PGP pgp;
} PTRLine;
typedef struct {
    int type;
    int vis;
    PGObjectList* pgl;
    PTRData* dp;
    PGP pgp;
} PTRMeshLines;
typedef struct {
    int type;
    int vis;
    PGObjectList* pgl;
    double x1;
    double x2;
    double x3;
    double x[5];
    char* str;
    PGP pgp;
} PTRString;
typedef struct {
    int type;
    int vis;
    PGObjectList* pgl;
    double x;
    double y;
    char* str;
    PGP pgp;
} PTRFixString;
typedef struct {
    int type;
    int vis;
    PGObjectList* pgl;
    PTRData* dp;
    double exp;
    PGP pgp;
} PTRArrows;
typedef struct {
    int type;
    int vis;
    PGObjectList* pgl;
    struct _PTRGraphList gpl;
} PTRGraphSet;
typedef struct {
    int type;
    int vis;
    PGObjectList* pgl;
    double x1;
    double x2;
    double x3;
    double x[5];
    char* str;
    PGP pgp;
    int ax;
    double lb_dist;
} PTRLabel;
typedef struct {
    int type;
    int vis;
    PGObjectList* pgl;
    int autoset;
    int ax;
    int num;
    int nx;
    double* x1;
    double* x2;
    double* x3;
    ScaleLabel* scl;
    double sc_dist;
    double scl_dist;
    double lb_dist;
    char form[64];
    PGP pgp;
} PTRScale;
typedef struct {
    int type;
    int vis;
    PGObjectList* pgl;
    int num;
    double x1[256];
    double x2[256];
    double x3[256];
    PGP pgp;
} PTRSclFrame;

#define PTRGraphLength 16

typedef union _PTRGraph {
    int type;
    PTRAnyGraph any;
    PTRPoints points;
    PTRLine line;
    PTRMeshLines meshlines;
    PTRString string;
    PTRFixString fixstring;
    PTRGraphSet graphset;
    PTRLabel label;
    PTRScale scale;
    PTRSclFrame frame;
    long pad[PTRGraphLength];
} PTRGraph;

typedef struct _PTRWindowList {
    struct _PTRWindow* wp;
    struct _PTRWindowList* next;
} PTRWindowList;

typedef struct _PTRWindow {
    PGWindow* pgw;
    struct _PTRWindow* parent;
    PTRWindowList wpl;
    PTRGraphList gpl;
    int ax1;
    int ax2;
    int ax3;
    int coord;
    PTRLabel* label[5];
    PTRScale* scale[5];
    PTRSclFrame* frame;
    int exact;
    double  min[5];
    double  max[5];
    double size[5];
} PTRWindow;


/* function prototypes */
/* library.c */

extern void PTRInitialize();
extern int SetUp(char* /* server */);
extern int PSSetUp();
extern void MoveWindow(
    PTRWindow* /* window */,
    double /* x (relative to parent) */,
    double /* y (relative to parent) */
);
extern void ResizeWindow(
    PTRWindow* /* window */,
    double /* width  (relative to parent) */,
    double /* height (relative to parent) */
);
extern void MoveResizeWindow(
    PTRWindow* /* window */,
    double /* x (relative to parent) */,
    double /* y (relative to parent) */,
    double /* width  (relative to parent) */,
    double /* height (relative to parent) */
);
extern void WindowRegion(
    PTRWindow* /* window */,
    int /* number of rows */,
    int /* number of columns */,
    int /* region number */
);
extern void ResizeFrame(
    PTRWindow* /* window */,
    unsigned int /* width */,
    unsigned int /* height */
);
extern void InitializeWindow(PTRWindow* /* window */);
extern PTRWindow* CreateFrame(
    int /* geometry x */,
    int /* geometry y */,
    unsigned int /* width */,
    unsigned int /* height */,
    char* /* title */
);
extern PTRWindow* CreateSimpleFrame();
extern PTRWindow* CreateWindow(
    PTRWindow* /* parent window */,
    double /* x (relative to parent) */,
    double /* y (relative to parent) */,
    double /* width (relative to parent) */,
    double /* height (relative to parent) */
);
extern PTRWindow* CreateSimpleWindow(
    PTRWindow* /* parent window */
);
extern void ShowWindow(PTRWindow* /* window to show */);
extern void ShowWindows(PTRWindow* /* window to show */);
extern void ShowAllWindows();
extern void PSLandscape(char* /* file name */);
extern void PSPortrait(char* /* file name */);
extern void PSPrint(PTRWindow* /* window */);
extern void ShowPage();
extern int DumpPTRWindowToXpm(PTRWindow*, char*);
extern void WaitKeyPress(
    char* /* key */,
    PTRWindow* /* window */,
    PTRWindow* /* frame */
);
extern void PendingKeyPress(
    char* /* key */,
    PTRWindow* /* window */,
    PTRWindow* /* frame */
);

/* data.c */

extern PTRData* CurrentData();
extern double ExtractV(
    PTRData* /* data */,
    int /* i */,
    int /* j */,
    int /* k */,
    int /* l */
);
extern int SearchNearestGrid(
    PTRData* /* data */,
    double /* x1 */,
    double /* x2 */,
    double /* x3 */,
    int* /* i */,
    int* /* j */,
    int* /* k */
);
extern int InterpolateVector(
    PTRData* /* data */,
    double /* x1 */,
    double /* x2 */,
    double /* x3 */,
    int /* vector length */,
    int[] /* vector indices */,
    double[] /* vector values */
);
extern PTRData* CreateGridData(
    int /* dimension */,
    int /* number of grid on x1 */,
    int /* number of grid on x2 */,
    int /* number of grid on x3 */,
    int /* length of vector */,
    double* /* x1 */,
    double* /* x2 */,
    double* /* x3 */,
    double*[] /* vector */
);
extern void ReadGridData(
    int /* dimension */,
    int /* number of grid on x1 */,
    int /* number of grid on x2 */,
    int /* number of grid on x3 */,
    int /* length of vector */,
    double* /* x1 */,
    double* /* x2 */,
    double* /* x3 */,
    double*[] /* vector */
);
extern PTRData* CreateSeqData(
    int /* dimension */,
    int /* number of points */,
    int /* length of vector */,
    double* /* x1 */,
    double* /* x2 */,
    double* /* x3 */,
    double*[] /* vector */
);
extern void ReadSeqData(
    int /* dimension */,
    int /* number of points */,
    int /* length of vector */,
    double* /* x1 */,
    double* /* x2 */,
    double* /* x3 */,
    double*[] /* vector */
);
extern void ReadData(
    int /* number of points */,
    double* /* x1 */,
    double* /* value */
);
extern PTRData* DuplicateData(PTRData*);
extern void SaveGridData(
    FILE* /* file pointer */,
    PTRData* /* data */
);
extern void SaveSeqData(
    FILE* /* file pointer */,
    PTRData* /* data */
);
extern void SaveData(
    char* /* file name */,
    PTRData* /* data */
);
extern void SaveDataFP(
    FILE* /* file */,
    PTRData* /* data */
);
extern PTRData* RestoreGridData(FILE* /* file pointer */);
extern PTRData* RestoreSeqData(FILE* /* file pointer */);
extern PTRData* RestoreData(char* /* file name */);
extern PTRData* RestoreDataFP(FILE* /* file */);
extern void LoadData(char* /* file name */);
extern PTRData* ExtractSeqDataI(PTRData*, int, int);
extern PTRData* ExtractSeqDataJ(PTRData*, int, int);
extern PTRData* ExtractSeqDataK(PTRData*, int, int);
extern int SearchStartPointK(
    PTRData* /* data */,
    int /* which component of the vector */,
    double /* value of the contour */,
    char* /* map */,
    int* /* i */,
    int* /* j */,
    int  /* k */,
    int* /* d */,
    int* /* e */
);
extern int SearchStartPointI(
    PTRData* /* data */,
    int /* which component of the vector */,
    double /* value of the contour */,
    char* /* map */,
    int  /* i */,
    int* /* j */,
    int* /* k */,
    int* /* d */,
    int* /* e */
);
extern int SearchStartPointJ(
    PTRData* /* data */,
    int /* which component of the vector */,
    double /* value of the contour */,
    char* /* map */,
    int* /* i */,
    int  /* j */,
    int* /* k */,
    int* /* d */,
    int* /* e */
);
extern int NextPointK(
    PTRData* /* data */,
    int /* which component of the vector */,
    double /* value of the contour */,
    char* /* map */,
    int* /* i */,
    int* /* j */,
    int  /* k */,
    int* /* d */,
    int* /* e */,
    double* /* x1 */,
    double* /* x2 */
);
extern int NextPointI(
    PTRData* /* data */,
    int /* which component of the vector */,
    double /* value of the contour */,
    char* /* map */,
    int  /* i */,
    int* /* j */,
    int* /* k */,
    int* /* d */,
    int* /* e */,
    double* /* x2 */,
    double* /* x3 */
);
extern int NextPointJ(
    PTRData* /* data */,
    int /* which component of the vector */,
    double /* value of the contour */,
    char* /* map */,
    int* /* i */,
    int  /* j */,
    int* /* k */,
    int* /* d */,
    int* /* e */,
    double* /* x3 */,
    double* /* x1 */
);
extern PTRData** ToContourDataK(
    PTRData* /* data */,
    int /* which component of the vector */,
    int /* index */,
    double /* value of the contour */,
    int* /* number of the data */
);
extern PTRData** ToContourDataI(
    PTRData* /* data */,
    int /* which component of the vector */,
    int /* index */,
    double /* value of the contour */,
    int* /* number of the data */
);
extern PTRData** ToContourDataJ(
    PTRData* /* data */,
    int /* which component of the vector */,
    int /* index */,
    double /* value of the contour */,
    int* /* number of the data */
);
extern PTRData** ToContourData(
    PTRData* /* data */,
    int /* which component of the vector */,
    char /* i, j, or k */,
    int /* index */,
    double /* value of the contour */,
    int* /* number of the data */
);
extern PTRData* ToStLineE_FB(
    PTRData* /* data */,
    int /* vector 1 */,
    int /* vector 2 */,
    int /* vector 3 */,
    double /* start x1 */,
    double /* start x2 */,
    double /* start x3 */,
    int /* number */
);
extern PTRData* ToStreamLine(
    PTRData* /* data */,
    int /* vector 1 */,
    int /* vector 2 */,
    int /* vector 3 */,
    double /* start x1 */,
    double /* start x2 */,
    double /* start x3 */,
    int /* number */
);
extern void StreamLineMode(
    int /* algorithm */,
    int /* direction */
);

/* graph.c */
extern PGObjectList* PointsToPGObjects(
    PTRWindow* /* window */,
    PTRPoints* /* points */
);
extern PGObjectList* LineToPGObjects(
    PTRWindow* /* window */,
    PTRLine* /* line */
);
#ifdef DEBUG
extern PGObjectList* MeshLinesToPGObjects(
    PTRWindow* /* window */,
    PTRMeshLines* /* meshlines */
);
#endif
extern PGObjectList* StringToPGObjects(
    PTRWindow* /* window */,
    PTRString* /* string */
);
extern PGObjectList* FixStringToPGObjects(
    PTRWindow* /* window */,
    PTRFixString* /* string */
);
#ifdef DEBUG
extern PGObjectList* ArrowTo...
extern PGObjectList* TopLabelToPGObjects(
    PTRWindow* /* window */,
    PTRString* /* label */
);
extern PGObjectList* X1LabelToPGObjects(
    PTRWindow* /* window */,
    PTRString* /* label */
);
extern PGObjectList* X2LabelToPGObjects(
    PTRWindow* /* window */,
    PTRString* /* label */
);
extern PGObjectList* X3LabelToPGObjects(
    PTRWindow* /* window */,
    PTRString* /* label */
);
extern PGObjectList* VLabelToPGObjects(
    PTRWindow* /* window */,
    PTRString* /* label */
);
extern void ScaleToPGLines1(
    PTRScale* /* PTRScale */,
    PGObjectList* /* graph list */,
    int /* number of scales */,
    ScaleLabel* /* scale labels */,
    double /* minimum */,
    double /* maximum */,
    double /* x3 value */
);
extern void ScaleToPGLines2(
    PTRScale* /* PTRScale */,
    PGObjectList* /* graph list */,
    int /* number of scales */,
    ScaleLabel* /* scale labels */,
    double /* minimum */,
    double /* maximum */,
    double /* x3 value */
);
#endif
extern PGObjectList* ScaleToPGObjects(
    PTRWindow* /* scales drawn on */,
    PTRScale* /* scale */
);
extern PGObjectList* SclFrameToPGObjects(
    PTRWindow* /* scales drawn on */,
    PTRSclFrame* /* scale frame */
);
extern PGObjectList* ToPGObjects(
    PTRWindow* /* graph is on this window */,
    PTRGraph* /* graph to PGPbject */
);
extern PTRGraphList* SearchGraph(
    PTRGraphList* /* top of the list */,
    PTRGraph* /* graph */
);
extern void RemovePGObjestsOf(
    PTRWindow* /* window */,
    PTRGraph* /* graph */
);
extern void RemoveGraph(
    PTRWindow* /* graph removed from */,
    PTRGraph* /* graph */
);
extern void ClearWindow(
    PTRWindow* /* window */
);
extern void ClearWindows(
    PTRWindow* /* window */
);
extern void UpdatePTRPoints(PTRWindow*, PTRPoints*);
extern void UpdatePTRLine(PTRWindow*, PTRLine*);
extern void UpdatePTRString(PTRWindow*, PTRString*);
extern void UpdatePTRLabel(PTRWindow*, PTRLabel*);
extern void UpdatePTRScale(PTRWindow*, PTRScale*);
extern void UpdatePTRSclFrame(PTRWindow*, PTRSclFrame*);
extern void UpdatePTRGProperties(
    PTRWindow* /* window */,
    PTRGraph* /* graph */
);
extern void ToAndAddPGObjects(
    PTRWindow* /* window */,
    PTRGraph* /* graph */
);
extern void AddPTRGraph(
    PTRWindow* /* graph add to */,
    PTRGraph* /* graph to be added */
);
extern void UpdatePTRGraph(
    PTRWindow* /* window */,
    PTRGraph* /* graph */
);
extern void UpdatePTRGraphs(
    PTRWindow* /* window */
);
extern void AddToGraphSet(
    PTRGraphSet* /* graph set */,
    PTRGraph* /* graph */
);
extern PTRGraphSet* CreateGraphSet();
extern PTRGraph* CreatePoints(
    PTRData* /* data structure */,
    int /* which component of the vector */,
    PGP
);
extern PTRGraph* DrawPoints(
    PTRWindow* /* window */,
    PTRData* /* data structure */,
    int /* which component of the vector */,
    PGP
);
extern PTRGraph* CreateLine(
    PTRData* /* data structure */,
    int /* which component of the vector */,
    PGP
);
extern PTRGraph* DrawLine(
    PTRWindow* /* window */,
    PTRData* /* data structure */,
    int /* which component of the vector */,
    PGP
);
extern PTRGraph* CreateMeshLines(
    PTRData* /* data structure */,
    int /* which component of the vector */,
    PGP
);
extern PTRGraph* DrawMeshLines(
    PTRWindow* /* window */,
    PTRData* /* data structure */,
    int /* which component of the vector */,
    PGP
);
extern PTRGraph* CreateString(
    double /* x1 */,
    double /* x2 */,
    double /* x3 */,
    double /* v */,
    char* /* string */,
    PGP
);
extern PTRGraph* DrawString(
    PTRWindow* /* window */,
    double /* x1 */,
    double /* x2 */,
    double /* x3 */,
    double /* v */,
    char* /* string */,
    PGP
);
extern PTRGraph* CreateFixString(
    double /* x */,
    double /* y */,
    char* /* string */,
    PGP
);
extern PTRGraph* DrawFixString(
    PTRWindow* /* window */,
    double /* x */,
    double /* y */,
    char* /* string */,
    PGP
);
extern PTRGraph* CreateContourLine(
    PTRData* /* field */,
    int /* which component of the vector */,
    char /* i, j, or k */,
    int /* index */,
    double /* value */,
    PGP
);
extern PTRGraph* DrawContourLine(
    PTRWindow* /* window */,
    PTRData* /* field */,
    int /* which component of the vector */,
    char /* i, j, or k */,
    int /* index */,
    double /* value */,
    PGP
);
#ifdef DEBUG
extern PTRGraph* CreateArrows(
    PTRData* /* data */,
    int /* vector 1 */,
    int /* vector 2 */,
    int /* vector 3 */,
    double /* expand */,
    PGP
);
extern PTRGraph* DrawArrows(
    PTRWindow* /* window */,
    PTRData* /* data */,
    int /* vector 1 */,
    int /* vector 2 */,
    int /* vector 3 */,
    double /* expand */,
    PGP
);
#endif
extern void RemoveAxLabel(PTRWindow*, int /* axis */);
extern PTRGraph* SetAxLabel(
    PTRWindow* /* window */,
    int /* axis */,
    char* /* label */,
    PGP
);
extern void RemoveTopLabel(PTRWindow* /* window */);
extern PTRGraph* SetTopLabel(
    PTRWindow* /* window */,
    char* /* label */,
    PGP
);
extern void RemoveX1Label(PTRWindow* /* window */);
extern PTRGraph* SetX1Label(
    PTRWindow* /* window */,
    char* /* label */,
    PGP
);
extern void RemoveX2Label(PTRWindow* /* window */);
extern PTRGraph* SetX2Label(
    PTRWindow* /* window */,
    char* /* label */,
    PGP
);
extern void RemoveX3Label(PTRWindow* /* window */);
extern PTRGraph* SetX3Label(
    PTRWindow* /* window */,
    char* /* label */,
    PGP
);
extern void RemoveVLabel(PTRWindow* /* window */);
extern PTRGraph* SetVLabel(
    PTRWindow* /* window */,
    char* /* label */,
    PGP
);
extern void RemoveXLabel(PTRWindow* /* window */);
extern PTRGraph* SetXLabel(
    PTRWindow* /* window */,
    char* /* label */,
    PGP
);
extern void RemoveYLabel(PTRWindow* /* window */);
extern PTRGraph* SetYLabel(
    PTRWindow* /* window */,
    char* /* label */,
    PGP
);
extern void RemoveZLabel(PTRWindow* /* window */);
extern PTRGraph* SetZLabel(
    PTRWindow* /* window */,
    char* /* label */,
    PGP
);
/* function.c */
extern int OverWrite(char* /* file name */);
extern PTRScale* CreatePTRScale(int, char*);
extern void ScaleStep(
    double /* range */,
    double* /* scale step */,
    double* /* label step */
);
extern int AutoScaleLabel(
    double /* minimum */,
    double /* maximum */,
    char* /* label form */,
    ScaleLabel** /* scale labels */
);
extern void RemoveSclFrame(PTRWindow*);
extern void SetSclFrame(PTRWindow*, PGP);
extern void SetAxScale(PTRWindow*, int, int, ScaleLabel[]);
extern void AddAxScale(PTRWindow*, int, double, char*);
extern void AutoAxScale(PTRWindow*, int, char*);
extern void SetX1Scale(PTRWindow*, int, ScaleLabel[]);
extern void AddX1Scale(PTRWindow*, double, char*);
extern void AutoX1Scale(PTRWindow*, char*);
extern void RemoveScale(PTRWindow*);
extern void SetScale(PTRWindow*, char*);
extern void UpdateScales(PTRWindow*);
extern void UpdateLabels(PTRWindow*);
extern void ExactRatio(
    PTRWindow* /* window */,
    int /* boolean */
);
extern void SetY_X(PTRWindow*, double);
extern void SetZ_X(PTRWindow*, double);
extern void SetPGWAxLimit(PTRWindow*, int, double, double);
extern void SetAxLimit(PTRWindow*, int, double, double);
extern double X1Min(PTRWindow*);
extern double X1Max(PTRWindow*);
extern void SetX1Limit(
    PTRWindow* /* window */,
    double /* x1 minimum */,
    double /* x1 maximum */
);
extern void SetX1Min(PTRWindow*, double);
extern void SetX1Max(PTRWindow*, double);
extern double X2Min(PTRWindow*);
extern double X2Max(PTRWindow*);
extern void SetX2Limit(
    PTRWindow* /* window */,
    double /* x2 minimum */,
    double /* x2 maximum */
);
extern void SetX2Min(PTRWindow*, double);
extern void SetX2Max(PTRWindow*, double);
extern double X3Min(PTRWindow*);
extern double X3Max(PTRWindow*);
extern void SetX3Limit(
    PTRWindow* /* window */,
    double /* x3 minimum */,
    double /* x3 maximum */
);
extern void SetX3Min(PTRWindow*, double);
extern void SetX3Max(PTRWindow*, double);
extern double VMin(PTRWindow*);
extern double VMax(PTRWindow*);
extern void SetVLimit(
    PTRWindow* /* window */,
    double /* v minimum */,
    double /* v maximum */
);
extern void SetVMin(PTRWindow*, double);
extern void SetVMax(PTRWindow*, double);
extern double XMin(PTRWindow*);
extern double XMax(PTRWindow*);
extern void SetXLimit(
    PTRWindow* /* window */,
    double /* x minimum */,
    double /* x maximum */
);
extern void SetXMin(PTRWindow*, double);
extern void SetXMax(PTRWindow*, double);
extern double YMin(PTRWindow*);
extern double YMax(PTRWindow*);
extern void SetYLimit(
    PTRWindow* /* window */,
    double /* y minimum */,
    double /* y maximum */
);
extern void SetYMin(PTRWindow*, double);
extern void SetYMax(PTRWindow*, double);
extern double ZMin(PTRWindow*);
extern double ZMax(PTRWindow*);
extern void SetZLimit(
    PTRWindow* /* window */,
    double /* z minimum */,
    double /* z maximum */
);
extern void SetZMin(PTRWindow*, double);
extern void SetZMax(PTRWindow*, double);
extern void SetCoord(
    PTRWindow* /* window */,
    int /* coord */
);
extern void SetX1Axis(PTRWindow*, int);
extern void SetX2Axis(PTRWindow*, int);
extern void SetX3Axis(PTRWindow*, int);
extern void SetAxis(PTRWindow*, int, int, int);
extern void SetPhi(PTRWindow*, double);
extern void SetTheta(PTRWindow*, double);
extern void SetMagnify(PTRWindow*, double);
extern void SetOriginX(PTRWindow*, double);
extern void SetOriginY(PTRWindow*, double);

/* util.c */
extern int IsFrameWindow(PTRWindow* /* window */);
extern PTRWindow* ParentWindow(PTRWindow* /* window */);
extern PTRWindow* ParentFrame(PTRWindow* /* window */);
extern char* AxLabelString(PTRWindow*, int);
extern char* TopLabelString(PTRWindow* /* window */);
extern char* X1LabelString(PTRWindow* /* window */);
extern char* X2LabelString(PTRWindow* /* window */);
extern char* X3LabelString(PTRWindow* /* window */);
extern char* VLabelString(PTRWindow* /* window */);
extern char* XLabelString(PTRWindow* /* window */);
extern char* YLabelString(PTRWindow* /* window */);
extern char* ZLabelString(PTRWindow* /* window */);
extern int HasScale(PTRWindow* /* window */);
extern int IsExactRatio(PTRWindow* /* window */);
extern void RaiseFrame(PTRWindow* /* window */);
extern int ExtractX1Range(
    PTRData* /* data */,
    double* /* minimum */,
    double* /* maximum */
);
extern int ExtractX2Range(
    PTRData* /* data */,
    double* /* minimum */,
    double* /* maximum */
);
extern int ExtractX3Range(
    PTRData* /* data */,
    double* /* minimum */,
    double* /* maximum */
);
extern int ExtractVRange(
    PTRData* /* data */,
    int /* which component */,
    double* /* minimum */,
    double* /* maximum */
);
extern int ExtractAxRange(int, PTRData*, int, double*, double*);
extern int ExtractXRange(
    PTRWindow* /* window */,
    PTRData* /* data */,
    int /* which component */,
    double* /* minimum */,
    double* /* maximum */
);
extern int ExtractYRange(
    PTRWindow* /* window */,
    PTRData* /* data */,
    int /* which component */,
    double* /* minimum */,
    double* /* maximum */
);
extern int ExtractZRange(
    PTRWindow* /* window */,
    PTRData* /* data */,
    int /* which component */,
    double* /* minimum */,
    double* /* maximum */
);

/* default.c */
extern void SetDefaultGeometry(
    int /* geometry x */,
    int /* geometry y */
);
extern int DefaultGMX();
extern int DefaultGMY();
extern void SetDefaultFrameSize(
    unsigned int /* width */,
    unsigned int /* height */
);
extern unsigned int DefaultWDX();
extern unsigned int DefaultWDY();
extern void SetDefaultWindowXY(
    double /* x */,
    double /* y */
);
extern double DefaultWindowX();
extern double DefaultWindowY();
extern void SetDefaultWindowSize(
    double /* width */,
    double /* height */
);
extern double DefaultWindowW();
extern double DefaultWindowH();
extern int SetColor(
    int /* red */,
    int /* green */,
    int /* blue */
);
extern int SetNamedColor(char* /* color name */);
extern int GetNamedColor(
    char* /* color name */,
    PGColor* /* PGColor structure */
);
extern int SetDefaultColors(
    char* /* default foreground */,
    char* /* default background */
);
extern void SetDefaultForeground(PGColor);
extern void SetDefaultBackground(PGColor);
extern PGColor DefaultForeground();
extern PGColor DefaultBackground();
extern void SetStreamLineNum(int /* number */);
extern int StreamLineNum();
extern void SetStreamLineLength(double /* length */);
extern double StreamLineLength();

/* free.c */
extern void FreeData(PTRData* /* data */);
extern void FreePoints(PTRPoints* /* points */);
extern void FreeLine(PTRLine* /* line */);
extern void FreeString(PTRString* /* string */);
extern void FreeFixString(PTRFixString* /* string */);
extern void FreeGraphSet(PTRGraphSet* /* graph set */);
extern void FreeScale(PTRScale* /* scale */);
extern void FreeGraph(PTRGraph* /* graph */);
extern void FreeGraphList(PTRGraphList* /* graph list */);
extern void FreeGraphLists(PTRGraphList* /* graph list */);

/* error.c */
extern void EnterFunc(char*);
extern void LeaveFunc();
extern char* FirstFunc();
extern void SetError(
    int /* level */,
    char* /* message */
);

/* flib.c */
extern char* scopy(char* /* string */, int /* length */);
extern int AddWindow(PTRWindow* /* window */);
extern PTRWindow* IDToWindow(int /* id */);
extern int AddData(PTRData* /* data */);
extern PTRData* IDToData(int /* id */);

/* version.c */
extern char* Pointer();
extern char* ShortVersion();
extern void EchoVersion();

#endif /* _PTR_H_ */
