#ifndef _PGW_H_
#define _PGW_H_
#include <X11/Xlib.h>

#define PGWINDOW "PGWindow-1.00b0"
#define USE_MEMORY_COPY


#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

#define RGB_TXT "/usr/X11R6/lib/X11/rgb.txt"

#define MAX_STR 256

#define PI 3.1416
#define SQRTPI 1.7725

#define BRKLINELEN 16

/* return values */
#define Unsuported  -101
#define NoWindow    -102

/* default values */
#define DFPOINT  21
#define DFRAD     5
#define DFARROW 0.3
#define DFARANGLE (0.25*PI)
#define DFCOLOR "default"


/* macro for postscript properties */

#define A3 1
#define A4 2
#define A5 3
#define B4 4
#define B5 5

#define LAND 1
#define PORT 2

#define PSMARGIN 0.05
#define A4SHORT 612
#define A4LONG  792

#define PSLINEWIDTH 1.0


/* default values */
#define UNITARROW  20
#define D3MARGIN  1.2


/* macro for fonts */
#define MAXPT 32
#define MINPT 6
#define PTSTEP 1

#define SUBPT 0.6

#define COUR 0
#define TIMS 1
#define HELV 2
#define SYMB 3
#define FMLY 4

#define RM 0
#define IT 1
#define SL 2

#define FONTNAMES {{"courier-medium-r", "courier-medium-o"}, {"times-medium-r", "times-medium-i"}, {"helvetica-medium-r", "helvetica-medium-o"}, {"symbol-*-*", "symbol-*-*"}}
#define PSFONTNAMES {{"Courier", "Courier-Italic"}, {"Times-Roman", "Times-Italic"}, {"Helvetica", "Helvetica-Oblique"}, {"Symbol", "Symbol"}}

#define RIGHT  1
#define LEFT   2
#define CENTER 0

#define R0     0
#define R90   90
#define R180 180
#define R270 270


/* macro for coordinates */
#define DIM2  2
#define DIM3  3
#define CAR   1
#define CYL   2

/* macro for varius graphic object types */

/* PGObject types */
#define PGPOINTS    1
#define PGLINE      2
#define PGCIRCLE    3
#define PGSTRING    4
#define PGFIXSTRING 5
#define PGARROWS    6

/* point types */
#define PTS 0
#define SQR 1
#define FSQR -1
#define CIR 2
#define FCIR -2

/* line types */
#define SOL 0
#define DOT 1
#define BRK 2

/* theta min-max */
#define Theta_Min (-0.5*PI)
#define Theta_Max ( 0.5*PI)


/* type definitions */

typedef struct _FontList {
    double pt;
    int fmly;
    int sl;
    double sub;
    struct _FontList* next;
} PGFontList;

typedef struct {
    int r;
    int g;
    int b;
    XColor xcolor;
} PGColor;

typedef struct {
    int ptype;
    int ltype;
    int flush;
    int rot;
    double rad;
    double pt;
    double arrow;
    double angle;
    double width;
    PGColor pgcl;
} PGP;

typedef struct {
    int type;
    double* x1;
    double* x2;
    double* x3;
    int num;
    PGP pgp;
} PGPoints;
typedef struct {
    int type;
    double* x1;
    double* x2;
    double* x3;
    int num;
    PGP pgp;
} PGLine;
typedef struct {
    int type;
    double x1;
    double x2;
    double x3;
    PGP pgp;
} PGCircle;
typedef struct {
    int type;
    double x1;
    double x2;
    double x3;
    char* str;
    PGP pgp;
} PGString;
typedef struct {
    int type;
    double x;
    double y;
    char* str;
    PGP pgp;
} PGFixString;
typedef struct {
    int type;
    double* x1;
    double* x2;
    double* x3;
    double* v1;
    double* v2;
    double* v3;
    int num;
    double exp;
    PGP pgp;
} PGArrows;

typedef union {
    int type;
    PGPoints pgpoints;
    PGLine pgline;
    PGString pgstring;
    PGFixString pgfixstring;
    PGArrows pgarrows;
} PGObject;


typedef struct _PGObjectList {
    PGObject* pg;
    struct _PGObjectList* next;
} PGObjectList;

typedef struct _PGWindowList {
    struct _PGWindow* pgw;
    struct _PGWindowList* next;
} PGWindowList;

typedef struct _PGWindow {
    struct _PGWindow* parent;
    PGWindowList pgwl;
    PGObjectList pgl;
    Window w;
    Pixmap p;
    GC gc;
    XEvent ev;
    double x;
    double y;
    double width;
    double height;
    unsigned int wdx;
    unsigned int wdy;
    double zerox;
    double zeroy;
    double x1vecx;
    double x1vecy;
    double x2vecx;
    double x2vecy;
    double x3vecx;
    double x3vecy;
    double radius;
    double mag;
    double orgx;
    double orgy;
    unsigned long bg;
    int dim;
    int coord;
    double margin;
    double x1min;
    double x1max;
    double x1size;
    double x2min;
    double x2max;
    double x2size;
    double x3min;
    double x3max;
    double x3size;
    double phi;
    double theta;
    double x2_x1;
    double x3_x1;
} PGWindow;


/* external variables */
extern PGP defpgp;


/* function prototypes */
/* pgwindow.c */

extern PGWindow* XWindowToPGWindow(
    PGWindow* /* window */,
    Window /* X window */
);
extern void AddPGWindow(
    PGWindow* /* parent frame */,
    PGWindow* /* window */
);
extern int PGGetColor(
    int /* red */,
    int /* green */,
    int /* blue */,
    PGColor* /* color structure */
);
extern int PGGetColors(
    int n,
    int[] /* red */,
    int[] /* green */,
    int[] /* blue */,
    PGColor[] /* color structure */
);
extern int PGGetNamedColor(
    char* /* color name */,
    PGColor* /* color structure */
);
extern int PGWSetUp(char* /* server */);
extern int PGWPSSetUp();
extern void MovePGWindow(
    PGWindow* /* window */,
    double /* x (relative to parent) */,
    double /* y (relative to parent) */
);
extern void ResizePGWindow(
    PGWindow* /* window */,
    double /* width  (relative to parent) */,
    double /* height (relative to parent) */
);
extern void ReconfigurePGWindow(
    PGWindow* /* window */
);
extern void MoveResizePGWindow(
    PGWindow* /* window */,
    double /* x (relative to parent) */,
    double /* y (relative to parent) */,
    double /* width  (relative to parent) */,
    double /* height (relative to parent) */
);
extern void ResizePGFrame(
    PGWindow* /* frame */,
    unsigned int /* width */,
    unsigned int /* height */
);
extern void PGWindowInitialize(PGWindow* /* window */);
extern PGWindow* CreatePGFrame(
    int /* geometry x */,
    int /* geometry y */,
    unsigned int /* width */,
    unsigned int /* height */,
    char* /* title */
);
extern PGWindow* CreatePGWindow(
    PGWindow* /* parent window */,
    double /* x (relative to parent) */,
    double /* y (relative to parent) */,
    double /* width  (relative to parent) */,
    double /* height (relative to parent) */
);
extern void ShowPGWindow(PGWindow* /* window to show */);
extern void ShowPGWindows(PGWindow* /* window to show */);
extern void ShowAllWindows();
extern int DumpPGWindowToXpm(PGWindow*, char*);
extern void RaisePGFrame(PGWindow* /* window */);
extern void EventHandling(PGWindow* /* window */);
extern void RotateEvent(PGWindow*, XEvent);
extern void MoveEvent(PGWindow*, XEvent);
extern void PGWMouseEventHandling(XEvent /* event */);
extern void HandleFrames();
extern int PGWWaitEvent(XEvent* /* event */);
extern int PGWWaitKeyPress(XEvent* /* event */);
extern int PGWPendingEvent(XEvent* /* event */);
extern int PGWPendingKeyPress(XEvent* /* event */);


/* pgwshow.c */

extern double TransScale(
    double /* width */,
    double /* height */,
    double /* radius to default paper */
);
extern void SetLineAttributes(
    PGWindow* /* window */,
    PGP /* PGP */
);
extern XFontStruct* GetXFont(
    double /* point */,
    int /* font family */,
    int /* slant type */
);
extern void DrawXCharacters(
    PGWindow* /* characters drawn on */,
    int /* pre-proccess */,
    int* /* x location on X window */,
    int* /* y location on X window */,
    int /* rotation */,
    char* /* characters to be drawn */,
    int /* length of characters */
);
extern int LabelCommand(
    char* /* command */,
    int* /* index */
);
extern void DrawXPGString(
    PGWindow* /* string drawn on */,
    char* /* string */,
    int /* flush */,
    int /* rotation */,
    double /* point */,
    int /* x location on X window */,
    int /* y location on X window */
);
extern double* LimitLocate(
    int /* num */,
    double* /* x */,
    double /* min */,
    double /* max */
);
extern void D2CarAxis(
    PGWindow* /* window */,
    char /* X or ps */,
    double /* width */,
    double /* height */
);
extern void D2CarTrans(
    PGWindow* /* window */,
    double /* x compornent of x1 vector */,
    double /* y compornent of x2 vector */,
    double /* x location of the origin */,
    double /* y location of the origin */,
    double* /* x1 in real coordinates */,
    double* /* x2 in real coordinates */,
    double* /* x on device */,
    double* /* y on device */,
    int /* number of points */
);
extern void D3CarAxis(
    PGWindow* /* window */,
    char /* X or ps */,
    double /* width */,
    double /* height */
);
extern void D3CarTrans(
    PGWindow* /* window */,
    double /* x compornent of x1 vector */,
    double /* y compornent of x1 vector */,
    double /* x compornent of x2 vector */,
    double /* y compornent of x2 vector */,
    double /* y compornent of x3 vector */,
    double /* x location of the origin */,
    double /* y location of the origin */,
    double* /* x1 in real coordinates */,
    double* /* x2 in real coordinates */,
    double* /* x3 in real coordinates */,
    double* /* x on device */,
    double* /* y on device */,
    int /* number of points */
);
extern void D2CylAxis(
    PGWindow* /* window */,
    char /* X or ps */,
    double /* width */,
    double /* height */
);
extern void D2CylTrans(
    PGWindow* /* window */,
    char /* X or ps */,
    double /* radius */,
    double /* x location of the origin */,
    double /* y location of the origin */,
    double* /* x1 in real coordinates */,
    double* /* x2 in real coordinates */,
    double* /* x on device */,
    double* /* y on device */,
    int /* number of points */
);
extern void D3CylAxis(
    PGWindow* /* window */,
    char /* X or ps */,
    double /* width */,
    double /* height */
);
extern void D3CylTrans(
    PGWindow* /* window */,
    char /* X or ps */,
    double /* radius */,
    double /* height */,
    double /* x location of the origin */,
    double /* y location of the origin */,
    double* /* x1 in real coordinates */,
    double* /* x2 in real coordinates */,
    double* /* x3 in real coordinates */,
    double* /* x on device */,
    double* /* y on device */,
    int /* number of points */
);
extern int AxisVector(
    PGWindow* /* window */,
    char /* X or ps */,
    double /* width */,
    double /* height */
);
extern int TransCoords(
    PGWindow* /* window */,
    char /* X or ps */,
    double* /* x1 in real coordinates */,
    double* /* x2 in real coordinates */,
    double* /* x3 in real coordinates */,
    double* /* x on device */,
    double* /* y on device */,
    int /* number of points */
);
extern void ShowPGPoints(
    PGWindow* /* window */,
    PGPoints* /* points object */
);
extern void ShowPGLine(
    PGWindow* /* window */,
    PGLine* /* line object */
);
extern void ShowPGString(
    PGWindow* /* window */,
    PGString* /* string object */
);
extern void ShowPGFixString(
    PGWindow* /* window */,
    PGFixString* /* string object */
);
extern void ShowPGArrows(
    PGWindow* /* window */,
    PGArrows* /* arrows object */
);

/* pgwprint.c */
extern void SetPSOrientation(int /* orientation */);
extern void SetPSPaperSize(int /* size */);
extern void SetPSCreator(char* /* creator's name */);
extern int PSShort();
extern int PSLong();
extern int PSSMargin();
extern int PSLMargin();
extern int PSWidth();
extern int PSHeight();
extern char* TimeStamp();
extern void MakeHeader();
extern void MakeProlog();
extern void PSSetForeground(PGColor /* color */);
extern void PSSetPoint(
    int /* point type */,
    double /* radius */
);
extern void PSSetDash(int /* line type */);
extern void PSSetLineWidth(double /* line width */);
extern void PSSetFont(
    double /* point */,
    int /* font family */,
    int /* slant type */
);
extern void DrawPSCharacters(
    int /* pre-proccess */,
    char* /* string */
);
extern void DrawPSPGString(
    PGWindow* /* window */,
    char* /* string */,
    int /* flush */,
    int /* rotation */,
    double /* point */
);
extern void PrintPGPoints(
    PGWindow* /* window */,
    PGPoints* /* points */,
    double /* paper width */,
    double /* paper height */
);
extern void PrintPGLine(
    PGWindow* /* window */,
    PGLine* /* line */,
    double /* paper width */,
    double /* paper height */
);
extern void PrintPGString(
    PGWindow* /* window */,
    PGString* /* string */,
    double /* paper width */,
    double /* paper height */
);
extern void PrintPGFixString(
    PGWindow* /* window */,
    PGFixString* /* string */,
    double /* paper width */,
    double /* paper height */
);
extern void PrintPGArrows(
    PGWindow* /* window */,
    PGArrows* /* arrows */,
    double /* paper width */,
    double /* paper height */
);
extern void PrintPGWindow(
    PGWindow* /* window */,
    double /* width */,
    double /* height */
);
extern int PGPSOpen(char* /* file name */);
extern int PGPrint(PGWindow* /* frame */);
extern int PGShowPage();

/* pgwlib.c */
extern void PushPGFontList(
    double /* point */,
    int /* family */,
    int /* slant */,
    double /* subscript */
);
extern int PopPGFontList();
extern PGFontList CurrentPGFont();
extern void DuplicateFontList();
extern double CurrentFontPoint();
extern void SetFontFamily(int /* family */);
extern void SetSlant(int /* slant */);
extern void MultipleFontPoint(double /* multiple */);
extern void AddSubscript(double /* add */);
extern void ClearFontList();
extern PGPoints* CreatePGPoints(
    int /* number of points */,
    double* /* x1 */,
    double* /* x2 */,
    double* /* x3 */,
    PGP
);
extern PGLine* CreatePGLine(
    int /* number of points */,
    double* /* x1 */,
    double* /* x2 */,
    double* /* x3 */,
    PGP
);
extern PGCircle* CreatePGCircle(
    double /* x1 */,
    double /* x2 */,
    double /* x3 */,
    PGP
);
extern PGString* CreatePGString(
    double /* x1 */,
    double /* x2 */,
    double /* x3 */,
    char* /* string */,
    PGP
);
extern PGFixString* CreatePGFixString(
    double /* x */,
    double /* y */,
    char* /* string */,
    PGP
);
extern PGArrows* CreatePGArrows(
    int /* number of points */,
    double* /* x1 */,
    double* /* x2 */,
    double* /* x3 */,
    double* /* v1 */,
    double* /* v2 */,
    double* /* v3 */,
    double /* expand */,
    PGP
);
extern void AddPGObject(
    PGWindow* /* graph object added to */,
    PGObject* /* graph object to be added */
);
extern void AddPGObjects(
    PGWindow* /* graph objects add to */,
    PGObjectList* /* graph list to be added */
);
extern void RemovePGObject(
    PGWindow* /* graph object removed from */,
    PGObject* /* graph object */
);
extern void RemovePGObjects(
    PGWindow* /* graph object removed from */,
    PGObjectList* /* graph list */
);
extern void PushToPGObjectList(PGObjectList*, PGObject*);
extern void SetPGWindowX1Limit(
    PGWindow* /* window */,
    double /* x1 minimum */,
    double /* x1 maximum */
);
extern void SetPGWindowX2Limit(
    PGWindow* /* window */,
    double /* x2 minimum */,
    double /* x2 maximum */
);
extern void SetPGWindowX3Limit(
    PGWindow* /* window */,
    double /* x3 minimum */,
    double /* x3 maximum */
);
extern void SetPGWindowX2_X1(
    PGWindow* /* window */,
    double /* ratio of x2 axis to x1 axis */
);
extern void SetPGWindowX3_X1(
    PGWindow* /* window */,
    double /* ratio of x3 axis to x1 axis */
);
extern void SetPGWindowCoord(
    PGWindow* /* window */,
    int /* coordinate */
);
extern void SetPGWindowDimension(
    PGWindow* /* window */,
    int /* dimension */
);
extern int Is3D(PGWindow* /* window */);
extern int IsPGFrame(PGWindow* /* window */);
extern PGWindow* ParentPGWindow(PGWindow* /* window */);
extern PGWindow* ParentPGFrame(PGWindow* /* window */);
extern void SetPhiAngle(PGWindow*, double);
extern void SetThetaAngle(PGWindow*, double);
extern void IncreasePhi(
    PGWindow* /* window */,
    double /* angle */
);
extern void IncreaseTheta(
    PGWindow* /* window */,
    double /* angle */
);
extern void SetOrgx(PGWindow*, double);
extern void SetOrgy(PGWindow*, double);
extern void HSVToRGB(double, double, double, int*, int*, int*);
extern int GrayColors(int, PGColor[]);

/* pgwfree.c */
extern void FreePGObjectList(PGObjectList* /* graph list */);
extern void FreePGObjectLists(PGObjectList* /* graph list */);
extern void FreePGPoints(PGPoints* /* points */);
extern void FreePGLine(PGLine* /* line */);
extern void FreePGString(PGString* /* string */);
extern void FreePGFixString(PGFixString* /* string */);
extern void FreePGArrows(PGArrows* /* arrows */);
extern void FreePGObject(PGObject* /* graph object */);

/* pgwdefaults.c */
extern void SetBaseLineWidth(double);
extern double BaseLineWidth();
extern void SetDefaultFontFamily(int);
extern int DefaultFontFamily();
extern void SetDefaultSlant(int);
extern int DefaultSlant();
extern void InitializePGP();
extern void SetDefaultLineType(int);
extern int DefaultLineType();
extern void SetDefaultPointType(int);
extern int DefaultPointType();
extern void SetDefaultPoint(double /* point */);
extern double DefaultPoint();
extern void SetDefaultRadius(double /* radius */);
extern double DefaultRadius();
extern void SetDefaultArrow(double /* arrow */);
extern double DefaultArrow();
extern void SetDefaultArrowAngle(double /* arrow angle */);
extern double DefaultArrowAngle();
extern void SetDefaultLineWidth(double /* line width */);
extern double DefaultLineWidth();
extern void SetDefaultColor(PGColor);
extern PGColor DefaultColor();
extern PGP GetPGP();

#endif /* _PGW_H_ */
