#include "../config.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "pointer.h"

#include <sys/types.h>



/* I/O control */
#ifdef USE_IOCTL
#ifdef PTR_BSD
#include <sys/ioctl.h>
struct sgttyb sgo;
#else
#include <termio.h>
struct termio tio;
#endif /* PTR_BSD */
#endif /* USE_IOCTL */





/* directory operation */
#ifdef USE_DIRECTORY
#ifdef PTR_BSD
#include <sys/dir.h>
#else
#include <dirent.h>
#define direct dirent
#endif /* PTR_BSD */
#endif /* USE_DIRECTORY */


/* use termcap */
#ifdef USE_TERMCAP
extern int tgetent(char*, char*);
extern int tgetnum(char*);
char* term=NULL;
#endif /* USE_TERMCAP */




char* comname=NULL;
char candfname[256];

int freeformat=FALSE;

typedef struct {
    int ps;
    char* fname;
} ArgSt;



/* raw mode */
#ifdef USE_IOCTL
#ifdef PTR_BSD
void rawon()
{
    struct sgttyb sgn;
    if(ioctl(STDIN, TIOCGETP, &sgo)==-1){
        CommandError(comname, "I/O control error.\n");
        exit(1);
    }
    sgn=sgo;
    sgn.sg_flags &= ~ECHO;
    sgn.sg_flags |= CBREAK;
    if(ioctl(STDIN, TIOCSETP, &sgn)==-1){
        CommandError(comname, "I/O control error.\n");
        exit(1);
    }
}
void rawoff()
{
    if(ioctl(STDIN, TIOCSETP, &sgo)==-1){
        CommandError(comname, "I/O control error.\n");
        exit(1);
    }
}
#else
void rawon()
{
    struct termio tin;
    if(ioctl(STDIN, TCGETA, &tio)==-1){
        CommandError(comname, "I/O control error.\n");
        exit(1);
    }
    tin=tio;
    tin.c_lflag &= ~ECHO;
    tin.c_lflag &= ~ICANON;
    tin.c_cc[VMIN] =1;
    tin.c_cc[VTIME]=0;
    if(ioctl(STDIN, TCSETA, &tin)==-1){
        CommandError(comname, "I/O control error.\n");
        exit(1);
    }
}
void rawoff()
{
    if(ioctl(STDIN, TCSETA, &tio)==-1){
        CommandError(comname, "I/O control error.\n");
        exit(1);
    }
}
#endif /* PTR_BSD */
#endif /* USE_IOCTL */




/* select system call */
#ifdef USE_SELECT
#include <sys/time.h>
int StdInPending()
{
    fd_set fds;
    struct timeval tv;
    static int n=0;
    FD_ZERO(&fds);
    FD_SET(STDIN, &fds);
    tv.tv_sec=0;
    if(n){
        tv.tv_usec=0;
    }else{
        tv.tv_usec=100000;
    }
    n=select(1, &fds, NULL, NULL, &tv);
    return n;
}
#endif /* USE_SELECT */


int GetTermCols()
{
    int ret=70;
#ifdef USE_TERMCAP
    int r;
    char buf[1024];
    if(term==NULL) return ret;
    if(tgetent(buf, term)<1) return ret;
    r=tgetnum("co");
    if(0<r) return r;
#endif /* USE_TERMCAP */
    return ret;
}


/* use file status system call */
#ifdef USE_STAT
#include <sys/stat.h>
char FileMark(char* path, char* fname)
{
    char fullname[512];
    struct stat sbuf;
    sprintf(fullname, "%s/%s", path, fname);
    if(lstat(fullname, &sbuf)==-1) return '\0';
    if((sbuf.st_mode & S_IFMT)==S_IFDIR){
        return '/';
    }else if((sbuf.st_mode & S_IFMT)==S_IFLNK){
        return '@';
    }else{
        if(sbuf.st_mode & S_IEXEC){
            return '*';
        }else if((sbuf.st_mode>>3) & S_IEXEC){
            return '*';
        }else if((sbuf.st_mode>>6) & S_IEXEC){
            return '*';
        }else{
            return ' ';
        }
    }
    return ' ';
}
int IsDirectory(char* path, char* fname)
{
    char fullname[512];
    struct stat sbuf;
    sprintf(fullname, "%s/%s", path, fname);
    if(stat(fullname, &sbuf)==-1) return FALSE;
    if((sbuf.st_mode & S_IFMT)==S_IFDIR) return TRUE;
    return FALSE;
}
#endif /* USE_STAT */

void EchoPrompt(){
    Element* el;
    if((el=VarNameToElement(V_Prompt))){
        printf("%s", ToString(el));
    }else{
        printf("%s", PROMPT);
    }
}

void InsertCharacter(int* i, char* str, char c)
{
    int j;
    for(j=strlen(str); *i<=j; j--) str[j+1]=str[j];
    str[*i]=c;
    printf("%s", str+(*i));  (*i)++;
    for(j=*i; str[j]!='\0'; j++) putchar('\b');
}
void DeleteCharacter(int* i, char* str)
{
    if(*i){
        int j;
        for(j=(*i); str[j-1]!='\0'; j++) str[j-1]=str[j];
        (*i)--;  putchar('\b');
        printf("%s \b", str+(*i));
        for(j=(*i); str[j]!='\0'; j++) putchar('\b');
    }else{
        putchar('\a');
    }
}
void CarriageReturn(int* i){
    for(; 0<*i; (*i)--) putchar('\b');
}
void ClearFollowings(int i, char* str){
    int j;
    for(j=i; j<strlen(str); j++) putchar(' ');
    for(; i<j; j--) putchar('\b');
    str[i]='\0';
}
void EndOfLine(int* i, char* str)
{
    while(str[*i]!='\0'){
        putchar(str[*i]);  (*i)++;
    }
}

/* directory operations */
#ifdef USE_DIRECTORY
/* if there is no system call operating directory */
#ifdef NO_SCANDIR
#ifdef USE_STAT
#ifndef DIRSIZ
#define DIRSIZ 64
#endif /* DIRSIZ */
#ifdef NO_OPENDIR
typedef struct {
    int d_fd;
} DIR;
DIR* OpenDir();
void CloseDir();
struct direct* ReadDir();
DIR* OpenDir(char* dir)
{
    struct stat stbuf;
    DIR* dp=(DIR*)malloc(sizeof(*dp));
    if((dp->d_fd=open(dir, 0))<0) return (0);
    if((fstat(dp->d_fd, &stbuf)<0)
       || ((stbuf.st_mode & S_IFDIR)==0)){
        CloseDir(dp);
        return (0);
    }
    return(dp);
}
void CloseDir(DIR* dp)
{
    close(dp->d_fd);
    free((char*)dp);
}
struct direct* ReadDir(DIR* dp)
{
    int ret;
    static struct direct dir;
    do{
        ret=read(dp->d_fd, &dir, sizeof(dir));
        if(ret!=sizeof(dir)) return (0);
    }while(dir.d_ino==0);
    return(&dir);
}
#else /* NO_OPENDIR */
#define OpenDir opendir
#define CloseDir closedir
#define ReadDir readdir
#endif /* NO_OPENDIR */
int ScanDir();
int AlphaSort();
int ScanDir(char* dir, struct direct*** nmptr,
            int (*slct)(), int (*compar)())
{
    int i, dirs;
    DIR* dirp;
    char** array;
    struct direct* ent;
    unsigned int nalloc=10, nentries=0;
    if((dirp=OpenDir(dir))==NULL) return (-1);
    array=(char**)malloc(nalloc*sizeof(char*));
    if(array==NULL) return (-1);
    while((ent=ReadDir(dirp))!=NULL){
        if(slct && (slct(ent)==0)) continue;
        if(nentries==nalloc){
            array=(char**)realloc(array,
                                  (nalloc+=10)*sizeof(char*));
            if(array==NULL) return (-1);
        }
        dirs=strlen(ent->d_name);
        if(DIRSIZ<dirs) dirs=DIRSIZ;
        array[nentries]=(char*)malloc((dirs+1)*sizeof(char));
        strncpy(array[nentries], ent->d_name, dirs);
        array[nentries][dirs]='\0';
        nentries++;
    }
    CloseDir(dirp);
    if((nentries+1)!=nalloc)
            array=(char**)realloc(array,
                                  (nentries+1)*sizeof(char*));
    if(compar!=0)
            qsort(array, nentries, sizeof(char**), compar);
    *nmptr=(struct direct**)malloc(nentries*sizeof(struct direct*));
    for(i=0; i<nentries; i++){
        (*nmptr)[i]=(struct direct*)
                         malloc(sizeof(struct direct)
                                +(strlen(array[i])+1)*sizeof(char));
        strcpy((*nmptr)[i]->d_name, array[i]);
    }
    return (nentries);
}
int AlphaSort(char** a, char** b)
{
    return (strcmp(*a, *b));
}
#endif /* USE_STAT */
#else /* NO_SCANDIR */
#define ScanDir scandir
#define AlphaSort alphasort
#endif /* NO_SCANDIR */
int FileCandSelection(struct direct* dir)
{
    int i, len;
    len=strlen(candfname);
    if(strlen(dir->d_name)<len) return FALSE;
    if(len==0){
        if(dir->d_name[0]=='.'){
            return FALSE;
        }else{
            return TRUE;
        }
    }
    for(i=0; i<len; i++){
        if(candfname[i]!=dir->d_name[i]) return FALSE;
    }
    return TRUE;
}
struct direct** FileCandidates(char* path, char* str, int* n)
{
    char* dir;
    struct direct** ret;
    extern int AlphaSort();
    dir=ExpandHomeDirectory(path);
    strcpy(candfname, str);
    *n=ScanDir(dir, &ret, FileCandSelection, AlphaSort);
    if(dir!=path) free(dir);
    return ret;
}
int FileCompletion(char* str)
{
    int i, j, l, n;
    char path[256];
    struct direct** cand;
    strcpy(path, str);
    for(l=strlen(path); 0<=l && path[l]!='/'; l--);
    l++;
    if(l==0){
        path[0]='.';  path[1]='/';  path[2]='\0';
    }else{
        path[l]='\0';
    }
    cand=FileCandidates(path, str+l, &n);
    if(n==0) return FALSE;
    if(n==1){
        strcpy(str+l, cand[0]->d_name);
        if(IsDirectory(path, cand[0]->d_name)){
            strcat(str, "/");
        }else{
            strcat(str, " ");
        }
        return TRUE;
    }
    j=strlen(str)-l;
    while(TRUE){
        char c=cand[0]->d_name[j];
        for(i=1; i<n; i++){
            if(cand[i]->d_name[j]!=c) return FALSE;
        }
        str[l+j]=c;  j++;
        str[l+j]='\0';
    }
}
void EchoFileCandidates(char* str)
{
    int i, l, lmax, n, col, cols, row, rows;
    char path[256];
    char fname[256];
    struct direct** cand;
    strcpy(path, str);
    for(l=strlen(path); 0<=l && path[l]!='/'; l--);
    l++;
    if(l==0){
        path[0]='.';  path[1]='/';  path[2]='\0';
    }else{
        path[l]='\0';
    }
    cand=FileCandidates(path, str+l, &n);
    if(n==0) return;
    lmax=0;
    for(i=0; i<n; i++){
        l=strlen(cand[i]->d_name);
        if(lmax<l) lmax=l;
    }
    cols=GetTermCols()/(lmax+4);
    if(cols<1) cols=1;
    rows=ceil((double)n/cols);
    for(row=0; row<rows; row++){
        for(col=0; col<cols; col++){
            i=rows*col+row;
            if(n<=i) break;
            sprintf(fname, "%s%c", cand[i]->d_name,
                    FileMark(path, cand[i]->d_name));
            printf("%-*s", lmax+1, fname);
            printf("   ");
        }
        putchar('\n');
    }
}
#endif /* USE_DIRECTORY */
char** CommandCandidates(char* str, int* n)
{
    *n=0;
    return NULL;
}
int CommandCompletion(char* str)
{
    int i, j, n;
    char** cand;
    cand=CommandCandidates(str, &n);
    if(n==0) return FALSE;
    if(n==1){
        strcpy(str, cand[0]);
        free(cand[0]);  free(cand);
        return TRUE;
    }
    j=strlen(str);
    while(TRUE){
        char c=cand[0][j];
        for(i=1; i<n; i++){
            if(cand[i][j]!=c){
                for(i=0; i<n; i++) free(cand[i]);
                free(cand);
                return FALSE;
            }
        }
        str[j]=c;  j++;
        str[j]='\0';
    }
}
void EchoCommandCandidates(char* str)
{
    printf("\n");
}
void TabCompletion(int* i, char* str)
{
    int k=0;  int j=*i;
    char buf[256];
    buf[0]='\0';
    j--;
    while(0<=j){
        if(str[j]==34 || str[j]==39){
            putchar('\a');
            return;
        }else if(str[j]==' ' || str[j]=='=' || str[j]==';'){
            break;
        }
        j--;
    }
    j++;
    for(; j<*i; j++){
        buf[k]=str[j];  k++;
    }
    buf[k]='\0';
    j-=k;
    while(j<0 && str[j]==' ') j--;
    if(j<0 || str[j]=='=' || str[j]==';'){
        if(CommandCompletion(buf)){
            for(; buf[k]!='\0'; k++)
                    InsertCharacter(i, str, buf[k]);
            InsertCharacter(i, str, ' ');
        }else{
            for(; buf[k]!='\0'; k++)
                    InsertCharacter(i, str, buf[k]);
            putchar('\a');
        }
    }else if(str[j]==34 || str[j]==39){
        putchar('\a');
    }else{
#ifdef USE_DIRECTORY
        if(FileCompletion(buf)){
            for(; buf[k]!='\0'; k++)
                    InsertCharacter(i, str, buf[k]);
#else
        if(FALSE){
#endif /* USE_DIRECTORY */
        }else{
            for(; buf[k]!='\0'; k++)
                    InsertCharacter(i, str, buf[k]);
            putchar('\a');
        }
    }
}
void CtrlDCandidates(int* i, char* str)
{
    int k=0;  int j=*i;
    char buf[256];
    buf[0]='\0';
    j--;
    while(0<=j){
        if(str[j]==34 || str[j]==39){
            putchar('\a');
            return;
        }else if(str[j]==' ' || str[j]=='=' || str[j]==';'){
            break;
        }
        j--;
    }
    j++;
    for(; j<*i; j++){
        buf[k]=str[j];  k++;
    }
    buf[k]='\0';
    j-=k;
    while(j<0 && str[j]==' ') j--;
    if(j<0 || str[j]=='=' || str[j]==';'){
        EchoCommandCandidates(buf);
    }else if(str[j]==34 || str[j]==39){
        putchar('\a');
        return;
    }else{
        putchar('\n');
#ifdef USE_DIRECTORY
        EchoFileCandidates(buf);
#endif /* USE_DIRECTORY */
    }
    EchoPrompt();
    printf("%s", str);
    for(j=*i; str[j]!='\0'; j++) putchar('\b');
}
void ReadStdIn(char* str, int n)
{
    int c, i=0;  int hist=0;
    char* buf;
    char key;
    buf=(char*)malloc(n*sizeof(char));
    rawon();
    str[i]='\0';
    EchoPrompt();
    while(TRUE){
#ifdef USE_SELECT
    while(!StdInPending()){
        fflush(stdout);
        PendingKeyPress(&key, NULL, NULL);
    }
#else
    fflush(stdout);
#endif /* USE_SELECT */
/*  c=getchar();*/
    read(STDIN, &key, sizeof(char)); /* for cut and paste */
    c=(int)key;
    if(c==EOF){
        SetQuitSignal();
        rawoff();
        free(buf);
        return;
    }
    if(c=='\n'){
        putchar('\n');
        StripSpaces(&str);
        if(str[0]!='\0')
                PushVariable(V_History, StringElement(str));
        rawoff();
        PendingKeyPress(&key, NULL, NULL);
        free(buf);
        return;
    }else if(c==CURSA && getchar()==CURSB){
        char dir;
        dir=getchar();
        if(dir==CURSUP){
            if(VariableLength(V_History)<=hist){
                putchar('\a');
            }else{
                Element* el;
                CarriageReturn(&i);
                ClearFollowings(i, str);
                if(hist==0) strcpy(buf, str);
                hist++;
                el=GetNthElement(hist, V_History);
                strcpy(str, ToString(el));
                printf("%s", str);
                i=strlen(str);
            }
        }else if(dir==CURSDW){
            if(hist==0){
                putchar('\a');
            }else{
                Element* el;
                CarriageReturn(&i);
                ClearFollowings(i, str);
                hist--;
                if(hist==0){
                    strcpy(str, buf);
                }else{
                    el=GetNthElement(hist, V_History);
                    strcpy(str, ToString(el));
                }
                printf("%s", str);
                i=strlen(str);
            }
        }else if(dir==CURSRT){
            if(i<strlen(str)){
                putchar(str[i]);
                i++;
            }else{
                putchar('\a');
            }
        }else if(dir==CURSLF){
            if(i){
                putchar('\b');
                i--;
            }else{
                putchar('\a');
            }
        }
    }else if(c=='\b' || c==DELETE){
        DeleteCharacter(&i, str);
    }else if(32<=c && c<=126){
        InsertCharacter(&i, str, c);
    }else if(c=='\t'){
        TabCompletion(&i, str);
    }else if(c==CTRL_D){
        if(str[i]==' ' || str[i]=='\0'){
            CtrlDCandidates(&i, str);
        }else{
            putchar(str[i]);  i++;
            DeleteCharacter(&i, str);
        }
    }else if(c==CTRL_A){
        CarriageReturn(&i);
    }else if(c==CTRL_E){
        EndOfLine(&i, str);
    }else if(c==CTRL_K){
        ClearFollowings(i, str);
    }else{
        putchar('\a');
    }
    }
    free(buf);
}
void ReadLine(char* str, int n, FILE* fp)
{
    int c, i=0;
    char key;
    if(fp!=stdin){
        fgets(str, n, fp);
        i=strlen(str)-1;
        if(str[i]=='\n') str[i]='\0';
        return;
    }
    rawon();
    str[i]='\0';
    while(TRUE){
#ifdef USE_SELECT
    while(!StdInPending()){
        fflush(stdout);
        PendingKeyPress(&key, NULL, NULL);
    }
#else
    fflush(stdout);
#endif /* USE_SELECT */
/*  c=getchar();*/
    read(STDIN, &key, sizeof(char)); /* for cut and paste */
    c=(int)key;
    if(c==EOF){
        rawoff();
        return;
    }
    if(c=='\n'){
        putchar('\n');
        rawoff();
        return;
    }else if(c==CURSA && getchar()==CURSB){
        char dir;
        dir=getchar();
        if(dir==CURSUP){
            putchar('\a');
        }else if(dir==CURSDW){
            putchar('\a');
        }else if(dir==CURSRT){
            if(i<strlen(str)){
                putchar(str[i]);
                i++;
            }else{
                putchar('\a');
            }
        }else if(dir==CURSLF){
            if(i){
                putchar('\b');
                i--;
            }else{
                putchar('\a');
            }
        }
    }else if(c=='\b' || c==DELETE){
        DeleteCharacter(&i, str);
    }else if(32<=c && c<=126){
        InsertCharacter(&i, str, c);
    }else if(c=='\t'){
        InsertCharacter(&i, str, c);
    }else if(c==CTRL_D){
        putchar(str[i]);  i++;
        DeleteCharacter(&i, str);
    }else if(c==CTRL_A){
        CarriageReturn(&i);
    }else if(c==CTRL_E){
        EndOfLine(&i, str);
    }else if(c==CTRL_K){
        ClearFollowings(i, str);
    }else{
        putchar('\a');
    }
    }
}

void ProccessFile(char* fname, FILE* fp)
{
    int i, n=0, nbuf=256;
    char buf[256];
    char fnameback[256];
    char* line;
    GetFileName(fnameback);
    SetFileName(fname);
    SetFirstError();
    fseek(fp, 0L, SEEK_SET);
    line=(char*)malloc(nbuf*sizeof(char));
    line[0]='\0';
    while(fgets(buf, 256, fp)){
        char* s;
        n++;
        if(nbuf<strlen(line)+256){
            nbuf+=256;  line=realloc(line, nbuf);
        }
        StripComments(buf);
        s=buf;
        StripSpaces(&s);
        i=strlen(s)-1;
        if(!freeformat && 0<i && s[i]=='\n'){
            if(s[i-1]=='\\'){
                s[i-1]=' ';
            }else if(s[i-1]!='{' && s[i-1]!=';'){
                s[i]=';';  s[i+1]='\n';  s[i+2]='\0';
            }
        }
        strcat(line, s);
    }
    if(line[strlen(line)-1]=='\n') line[strlen(line)-1]='\0';
    ProccessCommandLine(n, line);
    free(line);
    ClearQuitSignal();
    SetFileName(fnameback);
}
void ProccessStdIn()
{
    int i, nbuf=256;
    char str[256];
    char* buf;
    SetFileName("");
    buf=(char*)malloc(nbuf*sizeof(char));
    while(!QuitSignal()){
        char* s;
        buf[0]='\0';
        ReadStdIn(str, 256);
        if(QuitSignal()) break;
        StripComments(str);
        s=str;
        StripSpaces(&s);
        i=strlen(s)+strlen(buf);
        if(nbuf<i+2){
            nbuf+=256;
            buf=(char*)realloc(buf, nbuf*sizeof(char));
        }
        strcat(buf, s);
        if(freeformat){
            while(Parentheses(buf)
                  || (0<i && (buf[i-1]!=';' || buf[i-1]!='}'))){
                ReadLine(str, 256, stdin);
                StripComments(str);
                s=str;
                StripSpaces(&s);
                i=strlen(s)+strlen(buf);
                if(nbuf<i+2){
                    nbuf+=256;
                    buf=(char*)realloc(buf, nbuf*sizeof(char));
                }
                strcat(buf, s);
            }
        }else{
            while(Parentheses(buf)
                  || (0<i && buf[i-1]=='\\')){
                if(buf[i-1]=='\\'){
                    buf[i-1]=' ';
                }else if(buf[i-1]!='{' && buf[i-1]!=';'){
                    buf[i]=';';  buf[i+1]='\0';
                }
                ReadLine(str, 256, stdin);
                StripComments(str);
                s=str;
                StripSpaces(&s);
                i=strlen(s)+strlen(buf);
                if(nbuf<i+2){
                    nbuf+=256;
                    buf=(char*)realloc(buf, nbuf*sizeof(char));
                }
                strcat(buf, s);
            }
        }
        ProccessCommandLine(1, buf);
    }
    free(buf);
    ClearQuitSignal();
}
void ProccessResourceFile()
{
    char fname[256];
    FILE* fp;
    strcpy(fname, getenv("HOME"));
    strcat(fname, "/.ptrrc");
    if((fp=fopen(fname, "r"))){
        ProccessFile(fname, fp);
        fclose(fp);
    }
    if((fp=fopen("./.ptrrc", "r"))){
        ProccessFile("./.ptrrc", fp);
        fclose(fp);
    }
}

ArgSt CheckArguments(int argc, char* argv[])
{
    int i;
    ArgSt ret;
    ret.ps=FALSE;
    ret.fname=NULL;
    comname=argv[0];
    for(i=1; i<argc; i++){
        if(strcmp(argv[i], "--version")==0
           || strcmp(argv[i], "-v")==0){
            EchoVersion();
            exit(0);
        }else if(strcmp(argv[i], "--help")==0
                 || strcmp(argv[i], "-h")==0){
            EchoHelp();
            exit(0);
        }else if(strcmp(argv[i], "-ps")==0){
            ret.ps=TRUE;
        }else if(ret.fname==NULL){
            ret.fname=argv[i];
        }
    }
    return ret;
}

int main(int argc, char* argv[])
{
    ArgSt argst;
    argst=CheckArguments(argc, argv);
    /* SetErrorMessageLevel();*/
#ifdef USE_TERMCAP
    term=getenv("TERM");
#endif
    if(argst.ps){
        PSSetUp();
    }else if(!SetUp(NULL)){
        CommandError(comname, "Can't open display.\n");
        exit(1);
    }
    ProccessResourceFile();
    if(argst.fname){
        FILE* fp;
        if(!(fp=fopen(argst.fname, "r"))){
            char msg[256];
            sprintf(msg, E_FileError, argst.fname);
            CommandError(comname, msg);
            exit(1);
        }
        ProccessFile(argst.fname, fp);
    }else{
        ProccessStdIn();
    }
    exit(0);
}
