#include "../config.h"
#include <stdio.h>
#include <ptr.h>
#include <sys/types.h>


/* directory operation */
#ifdef USE_DIRECTORY
#ifdef PTR_BSD
#include <sys/dir.h>
#else
#include <dirent.h>
#define direct dirent
#endif /* PTR_BSD */
#endif /* USE_DIRECTORY */



#ifndef STDIN
#define STDIN 0
#endif

#define CURSA   27
#define CURSB   91
#define CURSUP  65
#define CURSDW  66
#define CURSRT  67
#define CURSLF  68
#define CTRL_A   1
#define CTRL_D   4
#define CTRL_E   5
#define CTRL_K  11
#define DELETE 127


/* macro */
#define PROMPT " > "

#ifndef None
#define None 0
#endif


/* macro for error message */
#define E_ComNotFound  "Command not found."
#define E_Unmatch      "Unmatched %c."
#define E_ExTermBF     "Extra term was found before %s."
#define E_ExTermAF     "Extra term was found after %s."
#define E_FewSents     "Too few sentences in %s."
#define E_BadArg       "\"%s\": Bad argument."
#define E_BadArgD      "\"%lf\": Bad argument."
#define E_BadArgs      "Bad arguments."
#define E_FewArgs      "Too few arguments."
#define E_NoFileSp     "No file was specified."
#define E_CannotRead   "line %d: \"%s\": Can't read, ignored."
#define E_ColumnLess   "line %d: Less than %d columns."
#define E_NoActWin     "No active window."
#define E_SameValue    "All values are %lf."
#define E_NoLTerm      "No term in the left side of the \"%s\"."
#define E_NoRTerm      "No term in the right side of the \"%s\"."
#define E_NoLineType   "\"%s\": No such line type."
#define E_NoPointType  "\"%s\": No such point type."
#define E_NoFont       "\"%s\": No such font family."
#define E_NoSlant      "\"%s\": No such slant type."
#define E_NoSuchVar    "\"%s\": No such variable."
#define E_NoDataType   "\"%s\": No such data type."
#define E_NoSuchAxis   "\"%s\": No such axis."
#define E_NoSuchCoord  "\"%s\": No such coordinate type."
#define E_NoByteSize   "\"%s\": No such Byte size type."
#define E_NoByteSizeSp "Byte size must be specified."
#define E_NoColName    "Color name or RGB value must be specified."
#define E_AllocCol     "Can't allocate color."
#define E_NoRegNum     "Region number must be specified."
#define E_LessRows     "Number of rows less than zero."
#define E_LessCols     "Number of columns less than zero."
#define E_NoDataSp     "No data was specified."
#define E_NoStrSp      "No string was specified."
#define E_BadVarName   "\"%s\": Invarid variable name."
#define E_NoMember     "Variable has no member named \"%s\"."
#define E_MemberSubst  "Substitution to member \"%s\" is not permitted."
#define E_NoIndex      "Index number must be specified."
#define E_NoIntIndex   "Index number isn\'t integer."
#define E_ZeroDiv      "Division by zero."
#define E_UnknownSuf   "Unknown file suffix."

/* file suffix */
#define F_UNKNOWN -1
#define F_NOSUF    0
#define F_GIF      1
#define F_XPM      2

/* variable names */
#define V_Argc     "$argc"
#define V_Argv     "$argv"
#define V_Prompt   "$prompt"
#define V_System   "$SYSTEM"
#define V_History  "$history"
#define V_Alias    "$aliases"
#define V_Sub      "$subroutines"
#define V_String   "$string0"
#define V_File     "$file0"
#define V_Data     "$data0"
#define V_Window   "$window0"

/* macro for comparison */
#define EQ   1
#define NE   2
#define LE   3
#define GE   4
#define LT   5
#define GT   6

/* macro for increment */
#define Incr 1
#define Decr 2

/* macro for math, substitusion */
#define Add  1
#define Sub  2
#define Mul  3
#define Div  4
#define Mod  5
#define Pow  6

/* macro for elements */
#define UNDEF   0
#define INT     1
#define DOUBLE  2
#define STR     3
#define VAR     4
#define COLOR   5
#define FP      6
#define DATA    7
#define WINDOW  8
#define IARRAY 11
#define DARRAY 12
#define LIST   40
#define BLOCK  50

/* macro for nodes */
#define N_Node      100
#define N_Element   101
/* variable */
#define N_Index     111
#define N_Key       112
#define N_Period    113
/* math */
#define N_Math      121
#define N_Not       131
#define N_Plus      132
#define N_Minus     133
#define N_Incr      134
/* comparison */
#define N_OR        141
#define N_AND       142
#define N_COMP      143
/* blocks */
#define N_If        151
#define N_While     152
#define N_For       153
/* others */
#define N_Command   161
#define N_Subst     181
#define N_Semicolon 171


/* type definitions */

typedef struct {
    int type;
    int tmp;
} Undef;
typedef struct {
    int type;
    int tmp;
    int val;
} Integer;
typedef struct {
    int type;
    int tmp;
    double val;
} Double;
typedef struct {
    int type;
    int tmp;
    char* val;
} String;
typedef struct {
    int type;
    int tmp;
    PGColor pgcl;
} Color;
typedef struct {
    int type;
    int tmp;
    char name[256];
    FILE* fp;
    char mode[4];
} FilePointer;
typedef struct {
    int type;
    int tmp;
    union _Element* x1;
    union _Element* x2;
    union _Element* x3;
    union _Element* v;
    PTRData* dp;
} Data;
typedef struct {
    int type;
    int tmp;
    PTRWindow* wp;
} WindowPointer;
typedef struct {
    int type;
    int tmp;
    int num;
    int* val;
} IArray;
typedef struct {
    int type;
    int tmp;
    int num;
    double* val;
} DArray;
typedef struct _List {
    int type;
    int tmp;
    struct _List* next;
    union _Element* el;
} List;
typedef struct _Node {
    int type;
    int tmp;
    int sub;
    struct _Node* left;
    struct _Node* right;
    union _Element* el;
} Node;
typedef struct {
    int type;
    int tmp;
    Node* node;
    struct _VariableList* vl;
} Block;

typedef union _Element {
    int type;
    Undef undef;
    Integer integer;
    Double real;
    String string;
    Color color;
    FilePointer file;
    Data data;
    WindowPointer window;
    IArray iarray;
    DArray darray;
    List list;
    Block block;
    Node node;
} Element;

typedef struct {
    char* sname;
    char* name;
} ShortWordList;
typedef struct {
    char* name;
    Element* (*func)(int, Element**);
    int var;
} CommandList;
typedef struct {
    char* name;
    int type;
} TypeList;

typedef struct _VariableList {
    char name[64];
    int global;
    Element* el;
    struct _VariableList* next;
} VariableList;

/* function prototypes */
/* main.c */
extern void rawon();
extern void rawoff();
#ifdef USE_SELECT
extern int StdInPending();
#endif
extern int GetTermCols();
extern char FileMark(char* /* path */, char* /* file name */);
extern int IsDirectory(char* /* path */, char* /* file name */);
extern void EchoPrompt();
extern void InsertCharacter(
    int* /* index */,
    char* /* string */,
    char /* character */
);
extern void DeleteCharacter(
    int* /* index */,
    char* /* string */
);
extern void CarriageReturn(int* /* index */);
extern void ClearFollowings(int /* index */, char* /* string */);
extern void EndOfLine(int* /* index */, char* /* string */);
#ifdef USE_DIRECTORY
extern int FileCandSelection(
    struct direct* /* directory */
);
extern struct direct** FileCandidates(
    char* /* path */,
    char* /* string */,
    int* /* number of candidates */
);
extern int FileCompletion(char* /* string */);
extern void EchoFileCandidates(char* /* string */);
#endif /* USE_DIRECTORY */
extern char** CommandCandidates(
    char* /* string */,
    int* /* number of candidates */
);
extern int CommandCompletion(char* /* string */);
extern void EchoCommandCandidates(char* /* string */);
extern void TabCompletion(int* /* index */, char* /* string */);
extern void CtrlDCandidates(int* /* index */, char* /* string */);
extern void ReadStdIn(char*, int);
extern void ReadLine(char*, int, FILE*);
extern void ProccessFile(
    char* /* file name */,
    FILE* /* file pointer */
);
extern void ProccessStdIn();
extern void ProccessResourceFile();
extern int main(int, char*[]);

/* parse.c */
extern void StripComments(char* /* string */);
extern void StripSpaces(char** /* string */);
extern void StripCharacters(char*, char** /* string */);
extern void StripNewLines(
    int* /* begining */,
    int* /* end */,
    char** /* string */
);
extern void TabToSpace(char* /* string */);
extern int Parentheses(char* /* string */);
extern int SearchFromEnd(
    int* /* line */,
    int /* index */,
    char* /* character(s) */,
    char* /* string */
);
extern int SearchFromBegining(
    int* /* line */,
    int /* index */,
    char* /* character(s) */,
    char* /* string */
);
extern Node* ParseToElement(int, int, char*);
extern Node* ParseList(int, int, char*);
extern Node* ParseFunction(int, int, char*);
extern Node* ParseTerm(int, int, char*);
extern Node* ParsePow(int, int, char*);
extern Node* ParseIncrement(int, int, char*);
extern Node* ParseAsterisk(int, int, char*);
extern Node* ParsePlus(int, int, char*);
extern Node* ParseCommand(int, int, char*);
extern Node* ParseSentence(int, int, char*);
extern Node* ParseCompare(int, int, char*);
extern Node* ParseEqual(int, int, char*);
extern Node* ParseAND(int, int, char*);
extern Node* ParseOR(int, int, char*);
extern Node* ParseSubstitiution(int, int, char*);
extern Node* ParseIf(int, int, char*);
extern Node* ParseWhile(int, int, char*);
extern Element* ParseForCondition(int, int, char*);
extern Node* ParseFor(int, int, char*);
extern Node* ParseSubroutine(int, int, char*);
extern Node* ParseBlock(int, int, char*);
extern Node* ParseSemicolon(int, int, char*);
extern int SeekVariable(char*, char* /* variable */, Element**);
extern Element* EvaluateString(Element* /* string element */);
extern Element* EvaluateList(Element*);
extern Element* ExpandVariable(Element* /* element */);
extern void ExpandVarArguments(List* /* list */);
extern void EvaluateAlias(List* /* list */);
extern List* EvaluateArguments(List* /* list */);
extern void InitializeBlock(List*, Block*);
extern Element* ExecuteBlock(List*, Block*);
extern Element* ExecuteSubroutine(char*, List*, Block*);
extern Element* ExecuteCommand(List* /* list */);
extern Element** PrepareElement(Node*);
extern Element* PrepareList(Node*);
extern void SetNodeVariable(Node*, Element*);
extern Element* IncrementNode(Node*);
extern Element* ExecuteNodeTree(Node*, Node* /* parent */);
extern void ProccessCommandLine(
    int /* line number */,
    char* /* command line */
);

/* variable.c */
extern void FreeNode(Node* /* node */);
extern void FreeTmpNode(Node* /* node */);
extern void FreeList(List* /* list */);
extern void FreeTmpList(List* /* list */);
extern void FreeElement(Element* /* element */);
extern void FreeTmpElement(Element* /* element */);
extern void FreeVariableList(VariableList*);
extern void SetTmpVal(Element*, int);
extern int IsSameElement(Element*, Element*);
extern int CompareElement(
    int /* operator */,
    Element*, Element*
);
extern Element* AddListList(List*, List*);
extern Element* AddListArray(List*, Element*);
extern Element* AddListOther(List*, Element*);
extern Element* AddListElements(Element*, Element*);
extern Element* AddArrayArray(Element*, Element*);
extern Element* AddArrayOther(Element*, Element*);
extern Element* AddArrayElements(Element*, Element*);
extern Element* AddStringElements(Element*, Element*);
extern Element* AddElements(Element*, Element*);
extern Element* SubtractElements(Element*, Element*);
extern Element* MultiplyElements(Element*, Element*);
extern Element* DivideElements(Element*, Element*);
extern Element* ModElements(Element*, Element*);
extern Element* PowElements(Element*, Element*);
extern Element* IncrElement(Element*, int);
extern Element* MathElement(Element*, double(*func)());
extern Element* MinVal(Element*);
extern Element* MaxVal(Element*);
extern Element* MinMaxVal(Element*);
extern Node* CopyNode(Node*);
extern Element* CopyElement(Element* /* element */);
extern int ToBoolean(Element* /* element */);
extern Element* UndefElement();
extern int ToInteger(Element* /* element */);
extern Element* IntegerElement(int /* int */);
extern double ToDouble(Element* /* element */);
extern Element* DoubleElement(double /* double */);
extern char* ToString(Element* /* element */);
extern Element* StringElement(char* /* string */);
extern Element* VariableElement(char* /* string */);
extern Element* ColorElement(PGColor);
extern Element* FileElement(
    FILE* /* file pointer */,
    char* /* file name */,
    char* /* mode */
);
extern Element* DataElement(PTRData*);
extern void UpdateDataElement(Data*);
extern Element* WindowElement(PTRWindow*);
extern IArray* ToIArray(Element*);
extern Element* IArrayElement(int /* number */, int* /* val */);
extern DArray* ToDArray(Element*);
extern Element* DArrayElement(int /* number */, double* /* val */);
extern Element* AllocDArray(int /* number */, double* /* val */);
extern List* ToList(Element* /* element */);
extern Element* ListElement(
    int /* number of elements */,
    Element*[] /* elements */
);
extern Block* ToBlock(Node*);
extern Element* BlockElement(Node*);
extern Node* AllocateNode();
extern Element* ToElement(char* /* string */);
extern Element* StructMember(Element*, char*);
extern Element* SeatchElement(
    List* /* list */,
    Element* /* element */
);
extern Element* PopFromList(List* /* list */);
extern void AddToList(
    List* /* list */,
    Element* /* element */
);
extern void PushToList(
    List* /* list */,
    Element* /* element */
);
extern List* CatList(List*, List*);
extern int ListLength(List* /* list */);
extern Element* NthElement(int, List*);
extern void SetNthElement(int, List*, Element*);
extern Element** PrepareNthElement(int, List*);
extern Element* NthValueI(int, IArray*);
extern Element* NthValueD(int, DArray*);
extern void SetNthValueI(int, IArray*, int);
extern void SetNthValueD(int, DArray*, double);
extern List* SearchKey(
    List* /* associated array */,
    Element* /* key */
);
extern Element* KeyToValue(
    List* /* associated array */,
    Element* /* key */
);
extern void AddToAsArray(
    List* /* associated array */,
    Element* /* key */,
    Element* /* value */
);
extern Element** PrepareKeyToValue(List*, Element* /* key */);
extern VariableList* AllocateVariable(char* /* name */);
extern Element* SearchVarElement(
    VariableList* /* variable list */,
    Element* /* element */
);
extern Element* VarNameToElement(char* /* name */);
extern Element** PrepareVariable(char* /* name */);
extern int VariableExists(char*, Element**);
extern void SetVariable(char* /* name */, Element*);
extern void PushVariable(char* /* name */, Element*);
extern int VariableLength(char* /* name */);
extern Element* GetNthElement(int, char* /* name */);
extern Element* GetAsArrayValue(
    char* /* name */,
    Element* /* key */
);
extern void AddToAsArrayVar(
    char* /* name */,
    Element* /* key */,
    Element* /* value */
);
extern void Globalize(char*);
extern void DefaultVariable(char* /* name */);
extern void ClearDefaultVariable();
extern void SetDefaultVariable(Element*);

/* command.c */
extern CommandList NameToCommand(char* /* command name */);
extern int IsVarCommand(CommandList /* command */);
extern Element* C_Alias(int, Element*[]);
extern Element* C_Axis(int, Element*[]);
extern Element* C_AxisRatio(int, Element*[]);
extern Element* C_BaseLineWidth(int, Element*[]);
extern Element* C_Break(int, Element*[]);
extern Element* C_Cd(int, Element*[]);
extern Element* C_Clear(int, Element*[]);
extern Element* C_Color(int, Element*[]);
extern Element* C_Continue(int, Element*[]);
extern Element* C_Contour(int, Element*[]);
extern Element* C_Coord(int, Element*[]);
extern Element* C_Cos(int, Element*[]);
extern Element* C_Data(int, Element*[]);
extern Element* C_DefaultFont(int, Element*[]);
extern Element* C_DefaultSlant(int, Element*[]);
extern Element* C_DefaultForeground(int, Element*[]);
extern Element* C_DefaultBackground(int, Element*[]);
extern Element* C_Draw(int, Element*[]);
extern Element* C_DrawString(int, Element*[]);
extern Element* C_Dump(int, Element*[]);
extern Element* C_Echo(int, Element*[]);
extern Element* C_ExactRatio(int, Element*[]);
extern Element* C_Exp(int, Element*[]);
extern Element* C_FGetS(int, Element*[]);
extern Element* C_File(int, Element*[]);
extern Element* C_FRead(int, Element*[]);
extern Element* C_FSeek(int, Element*[]);
extern Element* C_GetColor(int, Element*[]);
extern Element* C_GetS(int, Element*[]);
extern Element* C_Global(int, Element*[]);
extern Element* C_GrayColors(int, Element*[]);
extern Element* C_IsUndef(int, Element*[]);
extern Element* C_Limit(int, Element*[]);
extern Element* C_LineWidth(int, Element*[]);
extern Element* C_Log(int, Element*[]);
extern Element* C_Log10(int, Element*[]);
extern Element* C_Maximum(int, Element*[]);
extern Element* C_MeshLines(int, Element*[]);
extern Element* C_Minimum(int, Element*[]);
extern Element* C_MinMax(int, Element*[]);
extern Element* C_NewFrame(int, Element*[]);
extern Element* C_NewWindow(int, Element*[]);
extern Element* C_Open(int, Element*[]);
extern Element* C_Plot(int, Element*[]);
extern Element* C_PlotSize(int, Element*[]);
extern Element* C_Print(int, Element*[]);
extern Element* C_PrintF(int, Element*[]);
extern Element* C_PSLand(int, Element*[]);
extern Element* C_PSPort(int, Element*[]);
extern Element* C_Quit(int, Element*[]);
extern Element* C_Raise(int, Element*[]);
extern Element* C_Read(int, Element*[]);
extern Element* C_Redraw(int, Element*[]);
extern Element* C_Region(int, Element*[]);
extern Element* C_ResizeFrame(int, Element*[]);
extern Element* C_RestoreData(int, Element*[]);
extern Element* C_Return(int, Element*[]);
extern Element* C_SaveData(int, Element*[]);
extern Element* C_Scale(int, Element*[]);
extern Element* C_ShowPage(int, Element*[]);
extern Element* C_Sin(int, Element*[]);
extern Element* C_Split(int, Element*[]);
extern Element* C_SPrintF(int, Element*[]);
extern Element* C_Sqrt(int, Element*[]);
extern Element* C_System(int, Element*[]);
extern Element* C_Tan(int, Element*[]);
extern Element* C_ToInt(int, Element*[]);
extern Element* C_TopLabel(int, Element*[]);
extern Element* C_ToGif(int, Element*[]);
extern Element* C_ToXpm(int, Element*[]);
extern Element* C_Version(int, Element*[]);
extern Element* C_VLabel(int, Element*[]);
extern Element* C_VLimit(int, Element*[]);
extern Element* C_WaitKey(int, Element*[]);
extern Element* C_X1Label(int, Element*[]);
extern Element* C_X1Limit(int, Element*[]);
extern Element* C_X2Label(int, Element*[]);
extern Element* C_X2Limit(int, Element*[]);
extern Element* C_X3Label(int, Element*[]);
extern Element* C_X3Limit(int, Element*[]);
extern Element* C_XLabel(int, Element*[]);
extern Element* C_XLimit(int, Element*[]);
extern Element* C_YLabel(int, Element*[]);
extern Element* C_YLimit(int, Element*[]);
extern Element* C_ZLabel(int, Element*[]);
extern Element* C_ZLimit(int, Element*[]);

/* util.c */
extern void SetQuitSignal();
extern int QuitSignal();
extern void ClearQuitSignal();
extern void SetBreakSignal();
extern int BreakSignal();
extern void ClearBreakSignal();
extern void SetContinueSignal();
extern int ContinueSignal();
extern void ClearContinueSignal();
extern void SetReturnSignal();
extern int ReturnSignal();
extern void ClearReturnSignal();
extern int AnySignal();
extern int PTNToPT(
    char* /* name */,
    int* /* type */
);
extern int LTNToLT(
    char* /* name */,
    int* /* type */
);
extern int FontNameToFont(char*, int*);
extern char* FontToFontName(int);
extern int SlantNameToSlant(char*, int*);
extern char* SlantToSlantName(int);
extern int TypeToSize(char* /* type */);
extern char* TypeToName(int /* type */);
extern char* DataTypeToName(int /* data type */);
extern int NameToDataType(char* /* data type */);
extern char* AxisToName(int /* dim */);
extern int NameToAxis(char* /* dim */);
extern char* CoordToName(int /* coord */);
extern int NameToCoord(char* /* coord */);
extern int IsWord(char /* character */);
extern int IsVarWord(char /* character */);
extern int IsQuote(char /* character */);
extern int IsLeftParentheses(char /* character */);
extern int IsRightParentheses(char /* character */);
extern char OtherParentheses(char /* character */);
extern int IsFileMode(char* /* mode */);
extern int IsArray(Element* /* element */);
extern int IsNode(Element* /* element */);
extern int IsFunctionForm(char*, int);
extern int IsCommandForm(char*);
extern int IsIfBlock(char*);
extern int IsElseBlock(char*);
extern int IsWhileBlock(char*);
extern int IsForBlock(char*);
extern int IsSubroutine(char*);
extern int FileSuffix(char*);
extern char EscapedCharacter(char /* character */);
extern char* ExpandHomeDirectory(char* /* string */);
extern void LimitExpand(double* /* min */, double* /* max */);
extern List* Split(char /* key */, char* /* string */);
extern void SPrF(char*, char*, Element*);
extern char* SPrintF(char*, int, Element*[]);
extern void EchoError(char* /* message */);
extern void CommandError(
    char* /* command name */,
    char* /* message */
);
extern void ParseError(int /* line number */, char* /* message */);
extern void ExecError(char* /* message */);
extern int ParseErrorOccured();
extern void ClearParseError();
extern void SetFirstError();
extern void EchoHelp();
extern void SetFileName(char* /* file name */);
extern void GetFileName(char* /* file name */);
extern void SetSubName(char* /* file name */);
extern void GetSubName(char* /* file name */);
extern char* DefaultString();
extern FILE* DefaultFile();
extern Element* DefaultFileElement();
extern PTRWindow* ActiveWindow();
extern PTRData* DefaultData();
