#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <ptr.h>
#include "pointer.h"



VariableList global={"", 1, NULL, NULL};
VariableList* crvl=&global;

char dfvname[16]="";



/* elements */

void FreeNode(Node* node){
    if(node==NULL) return;
    if(node->left)  FreeElement((Element*)(node->left));
    if(node->right) FreeElement((Element*)(node->right));
    if(node->el) FreeElement(node->el);
    free(node);
}
void FreeTmpNode(Node* node){
    if(node==NULL) return;
    if(node->left)  FreeTmpElement((Element*)(node->left));
    if(node->right) FreeTmpElement((Element*)(node->right));
    if(node->el) FreeTmpElement(node->el);
    free(node);
}
void FreeList(List* list){
    Element* el;
    if(list==NULL) return;
    while((el=PopFromList(list))) FreeElement(el);
    free(list);
}
void FreeTmpList(List* list){
    Element* el;
    if(list==NULL) return;
    if(list->tmp==FALSE) return;
    while((el=PopFromList(list))) FreeTmpElement(el);
    free(list);
}
void FreeElement(Element* el){
    if(el==NULL) return;
    if(el->type==LIST){
        FreeList((List*)el);
        return;
    }else if(IsNode(el)){
        FreeNode((Node*)el);
        return;
    }else if(el->type==STR){
        free(el->string.val);
    }else if(el->type==VAR){
        free(el->string.val);
    }else if(el->type==FP){
        Element* buf;
        buf=CopyElement(el);
        el->file.fp=NULL;
        if(SearchVarElement(crvl, buf)){
        }else if(&global==crvl || !SearchVarElement(&global, buf)){
            fclose(buf->file.fp);
        }
        free(buf);
    }else if(el->type==DATA){
        UpdateDataElement((Data*)el);
        FreeData(el->data.dp);
        free(el->data.x1);  free(el->data.x2);  free(el->data.x3);
        free(el->data.v);
    }else if(el->type==IARRAY){
        free(el->iarray.val);
    }else if(el->type==DARRAY){
        free(el->darray.val);
    }else if(el->type==BLOCK){
        FreeNode(el->block.node);
        FreeVariableList(el->block.vl);
    }
    free(el);
}
void FreeTmpElement(Element* el){
    if(el==NULL) return;
    if(el->undef.tmp==FALSE) return;
    if(el->type==LIST){
        FreeTmpList((List*)el);
        return;
    }else if(IsNode(el)){
        FreeTmpNode((Node*)el);
        return;
    }else if(el->type==STR){
        free(el->string.val);
    }else if(el->type==VAR){
        free(el->string.val);
    }else if(el->type==FP){
        Element* buf;
        buf=CopyElement(el);
        el->file.fp=NULL;
        if(SearchVarElement(crvl, buf)){
        }else if(&global==crvl || !SearchVarElement(&global, buf)){
            fclose(buf->file.fp);
        }
        free(buf);
    }else if(el->type==DATA){
        UpdateDataElement((Data*)el);
        FreeData(el->data.dp);
        free(el->data.x1);  free(el->data.x2);  free(el->data.x3);
        free(el->data.v);
    }else if(el->type==IARRAY){
        free(el->iarray.val);
    }else if(el->type==DARRAY){
        free(el->darray.val);
    }else if(el->type==BLOCK){
        FreeTmpNode(el->block.node);
        FreeVariableList(el->block.vl);
    }
    free(el);
}

void FreeVariableList(VariableList* vlist)
{
    VariableList* vl;
    if(vlist==NULL) return;
    while(vlist->next){
        vl=vlist->next;
        FreeElement(vl->el);
        vlist->next=vl->next;
        free(vl);
    }
}

void SetTmpVal(Element* el, int val)
{
    if(el==NULL) return;
    if(el->type==LIST){
        List* le;
        le=el->list.next;
        while(le){
            if(le->el) le->el->undef.tmp=val;
            le=le->next;
        }
    }else if(el->type==DATA){
        if(el->data.x1) el->data.x1->undef.tmp=val;
        if(el->data.x2) el->data.x2->undef.tmp=val;
        if(el->data.x3) el->data.x3->undef.tmp=val;
        if(el->data.v)  el->data.v ->undef.tmp=val;
    }
    el->undef.tmp=val;
}

/* element operations */
Element* CatStrings(Element* el1, char c, Element* el2)
{
    char* s1;  char* s2;
    char* buf;
    Element* ret;
    s1=ToString(el1);  s2=ToString(el2);
    buf=(char*)malloc((strlen(s1)+strlen(s2)+3)*sizeof(char));
    sprintf(buf, "%s%c%s", s1, c, s2);
    if(el1->type!=STR) free(s1);
    if(el2->type!=STR) free(s2);
    ret=StringElement(buf);
    free(buf);
    return ret;
}
int IsSameElement(Element* el1, Element* el2)
{
    if(!el1 || !el2) return FALSE;
    if(el1->type!=el2->type) return FALSE;
    switch(el1->type){
      case INT:
        return (el1->integer.val==el2->integer.val);
        break;
      case DOUBLE:
        return (el1->real.val==el2->real.val);
        break;
      case STR:
        return (strcmp(el1->string.val, el2->string.val)==0);
        break;
      case COLOR:
        return (el1->color.pgcl.r==el2->color.pgcl.r
                && el1->color.pgcl.g==el2->color.pgcl.g
                && el1->color.pgcl.b==el2->color.pgcl.b);
        break;
      case FP:
        return (el1->file.fp==el2->file.fp);
        break;
      case DATA:
        return (el1->data.dp==el2->data.dp);
        break;
      case WINDOW:
        return (el1->window.wp==el2->window.wp);
        break;
      case IARRAY:
        return (memcmp((char*)(el1->iarray.val),
                       (char*)(el2->iarray.val),
                       sizeof(int)/sizeof(char)
                       *el1->iarray.num)==0);
        break;
      case DARRAY:
        return (memcmp((char*)(el1->darray.val),
                       (char*)(el2->darray.val),
                       sizeof(double)/sizeof(char)
                       *el1->darray.num)==0);
        break;
      case LIST:
        return FALSE;
        break;
    }
    return FALSE;
}
int CompareElements(int op, Element* el1, Element* el2)
{
    long ival;
    double val;
    if(!el1 || !el2) return FALSE;
    if(el1->type!=el2->type) return FALSE;
    switch(el1->type){
      case INT:
        ival=(long)(el1->integer.val-el2->integer.val);
        break;
      case DOUBLE:
        val=el1->real.val-el2->real.val;
        if(val==0.0){
            ival=0;
        }else if(0.0<val){
            ival=1;
        }else{
            ival=-1;
        }
        break;
      case STR:
        ival=(long)strcmp(el1->string.val, el2->string.val);
        break;
      case COLOR:
        ival=(long)(el1->color.pgcl.r-el2->color.pgcl.r);
        break;
      case FP:
        ival=(long)(el1->file.fp)-(long)(el2->file.fp);
        break;
      case DATA:
        ival=(long)(el1->data.dp)-(long)(el2->data.dp);
        break;
      case WINDOW:
        ival=(long)(el1->window.wp)-(long)(el2->window.wp);
        break;
      case IARRAY:
        if(el1->iarray.num!=el2->iarray.num){
            ival=FALSE;
        }else{
            ival=strncmp((char*)(el1->iarray.val),
                         (char*)(el2->iarray.val),
                         el1->iarray.num);
        }
        break;
      case DARRAY:
        if(el1->darray.num!=el2->darray.num){
            ival=FALSE;
        }else{
            ival=strncmp((char*)(el1->darray.val),
                        (char*)(el2->darray.val),
                        el1->darray.num);
        }
        break;
      case LIST:
        return FALSE;
        break;
      default:
        return FALSE;
    }
    if(ival==0){
        switch(op){
          case EQ:
          case LE:
          case GE:
            return TRUE;
            break;
          default:
            return FALSE;
        }
    }else if(0<ival){
        switch(op){
          case NE:
          case GE:
          case GT:
            return TRUE;
            break;
          default:
            return FALSE;
        }
    }else{
        switch(op){
          case NE:
          case LE:
          case LT:
            return TRUE;
            break;
          default:
            return FALSE;
        }
    }
}
Element* AddListList(List* list1, List* list2)
{
    List* ret;
    List* le1;
    List* le2;
    ret=ToList(NULL);
    if(ListLength(list1)<ListLength(list2)){
        List* l;
        l=list1;  list1=list2;  list2=l;
    }
    le1=list1->next;
    le2=list2->next;
    while(le1){
        if(le2){
            AddToList(ret, AddElements(le1->el, le2->el));
            le2=le2->next;
        }else{
            AddToList(ret, CopyElement(le1->el));
        }
        le1=le1->next;
    }
    return (Element*)ret;
}
Element* AddListArray(List* list, Element* el)
{
    int i=0;
    List* ret;
    List* le;
    Element* e;
    ret=ToList(NULL);
    le=list->next;
    if(el->type==IARRAY){
        while(le){
            if(i<el->iarray.num){
                e=IntegerElement(el->iarray.val[i]);
                AddToList(ret, AddElements(le->el, e));
                FreeElement(e);  i++;
            }else{
                AddToList(ret, CopyElement(le->el));
            }
            le=le->next;
        }
        for(; i<el->iarray.num; i++){
            AddToList(ret, IntegerElement(el->iarray.val[i]));
        }
    }else if(el->type==DARRAY){
        while(le){
            if(i<el->darray.num){
                e=DoubleElement(el->darray.val[i]);
                AddToList(ret, AddElements(le->el, e));
                FreeElement(e);  i++;
            }else{
                AddToList(ret, CopyElement(le->el));
            }
            le=le->next;
        }
        for(; i<el->darray.num; i++){
            AddToList(ret, DoubleElement(el->darray.val[i]));
        }
    }
    return (Element*)ret;
}
Element* AddListOther(List* list, Element* el)
{
    List* le;
    List* ret;
    ret=ToList(NULL);
    le=list->next;
    while(le){
        AddToList(ret, AddElements(le->el, el));
        le=le->next;
    }
    return (Element*)ret;
}
Element* AddListElements(Element* el1, Element* el2)
{
    if(el1->type==el2->type){
        return AddListList((List*)el1, (List*)el2);
    }else if(el1->type==IARRAY || el1->type==DARRAY){
        return AddListArray((List*)el2, el1);
    }else if(el2->type==IARRAY || el2->type==DARRAY){
        return AddListArray((List*)el1, el2);
    }else if(el1->type!=LIST){
        return AddListOther((List*)el2, el1);
    }else{
        return AddListOther((List*)el1, el2);
    }
}
Element* AddArrayArray(Element* el1, Element* el2)
{
    int i;
    Element* ret;
    if(el1->iarray.num<el2->iarray.num){
        Element* e;
        e=el1;  el1=el2;  el2=el1;
    }
    if(el1->type==IARRAY && el2->type==IARRAY){
        int* val;
        val=(int*)malloc(el1->iarray.num*sizeof(int));
        for(i=0; i<el2->iarray.num; i++)
                val[i]=el1->iarray.val[i]+el2->iarray.val[i];
        for(; i<el1->iarray.num; i++)
                val[i]=el1->iarray.val[i];
        ret=IArrayElement(el1->iarray.num, val);
        free(val);
    }else if(el1->type==DARRAY && el2->type==DARRAY){
        double* val;
        val=(double*)malloc(el1->darray.num*sizeof(double));
        for(i=0; i<el2->darray.num; i++)
                val[i]=el1->darray.val[i]+el2->darray.val[i];
        for(; i<el1->darray.num; i++)
                val[i]=el1->darray.val[i];
        ret=DArrayElement(el1->darray.num, val);
        free(val);
    }else if(el1->type==DARRAY && el2->type==IARRAY){
        double* val;
        val=(double*)malloc(el1->darray.num*sizeof(double));
        for(i=0; i<el2->iarray.num; i++)
                val[i]=el1->darray.val[i]+el2->iarray.val[i];
        for(; i<el1->darray.num; i++)
                val[i]=el1->darray.val[i];
        ret=DArrayElement(el1->darray.num, val);
        free(val);
    }else if(el1->type==IARRAY && el2->type==DARRAY){
        double* val;
        val=(double*)malloc(el1->iarray.num*sizeof(double));
        for(i=0; i<el2->darray.num; i++)
                val[i]=el1->iarray.val[i]+el2->darray.val[i];
        for(; i<el1->iarray.num; i++)
                val[i]=el1->iarray.val[i];
        ret=DArrayElement(el1->iarray.num, val);
        free(val);
    }
    return ret;
}
Element* AddArrayOther(Element* el1, Element* el2)
{
    int i;
    Element* ret;
    if(el1->type==IARRAY && el2->type==DOUBLE){
        double* val;
        val=(double*)malloc(el1->iarray.num*sizeof(double));
        for(i=0; i<el1->iarray.num; i++)
                val[i]=el1->iarray.val[i]+ToDouble(el2);
        ret=DArrayElement(el1->iarray.num, val);
        free(val);
    }else if(el1->type==IARRAY){
        int* val;
        val=(int*)malloc(el1->iarray.num*sizeof(int));
        for(i=0; i<el1->iarray.num; i++)
                val[i]=el1->iarray.val[i]+ToInteger(el2);
        ret=IArrayElement(el1->iarray.num, val);
        free(val);
    }else{
        double* val;
        val=(double*)malloc(el1->darray.num*sizeof(double));
        for(i=0; i<el1->darray.num; i++)
                val[i]=el1->darray.val[i]+ToDouble(el2);
        ret=DArrayElement(el1->darray.num, val);
        free(val);
    }
    return ret;
}
Element* AddArrayElements(Element* el1, Element* el2)
{
    if(IsArray(el1) && IsArray(el2)){
        return AddArrayArray(el1, el2);
    }else if(!IsArray(el1)){
        return AddArrayOther(el2, el1);
    }else{
        return AddArrayOther(el1, el2);
    }
}
Element* AddElements(Element* el1, Element* el2)
{
    if(!el1 || !el2) return NULL;
/*    if(!el1) return CopyElement(el2);
    if(!el2) return CopyElement(el1);*/
    if(el1->type==STR || el2->type==STR){
        return CatStrings(el1, '+', el2);
    }else if(el1->type==LIST || el2->type==LIST){
        return AddListElements(el1, el2);
    }else if(IsArray(el1) || IsArray(el2)){
        return AddArrayElements(el1, el2);
    }else if(el1->type==DOUBLE || el2->type==DOUBLE){
        return DoubleElement(ToDouble(el1)+ToDouble(el2));
    }else if(el1->type==INT || el1->type==INT){
        return IntegerElement(ToInteger(el1)+ToInteger(el2));
    }else{
        return NULL;
    }
}
Element* SubtractElements(Element* el1, Element* el2)
{
    if(!el1 || !el2) return NULL;
/*    if(!el1) return MultiplyElements(IntegerElement(-1), el2);
    if(!el2) return CopyElement(el1);*/
    if(el1->type==STR || el2->type==STR){
        return CatStrings(el1, '-', el2);
    }else if(el1->type==LIST || el2->type==LIST){
/*        return AddListElements(el1, el2);*/
    }else if(IsArray(el1) || IsArray(el2)){
/*        return AddArrayElements(el1, el2);*/
    }else if(el1->type==DOUBLE || el2->type==DOUBLE){
        return DoubleElement(ToDouble(el1)-ToDouble(el2));
    }else if(el1->type==INT || el1->type==INT){
        return IntegerElement(ToInteger(el1)-ToInteger(el2));
    }else{
        return NULL;
    }
    return NULL;
}
Element* MultiplyElements(Element* el1, Element* el2)
{
    if(!el1 || !el2) return NULL;
/*    if(!el1 || !el2) return IntegerElement(0);*/
    if(el1->type==STR || el2->type==STR){
        return CatStrings(el1, '*', el2);
    }else if(el1->type==LIST || el2->type==LIST){
/*        return AddListElements(el1, el2);*/
    }else if(IsArray(el1) || IsArray(el2)){
/*        return AddArrayElements(el1, el2);*/
    }else if(el1->type==DOUBLE || el2->type==DOUBLE){
        return DoubleElement(ToDouble(el1)*ToDouble(el2));
    }else if(el1->type==INT || el1->type==INT){
        return IntegerElement(ToInteger(el1)*ToInteger(el2));
    }else{
        return NULL;
    }
    return NULL;
}
Element* DivideElements(Element* el1, Element* el2)
{
    if(!el1 || !el2) return NULL;
/*    if(!el2) return NULL;
    if(!el1) return IntegerElement(0);*/
    if(el1->type==STR || el2->type==STR){
        return CatStrings(el1, '/', el2);
    }else if(el1->type==LIST || el2->type==LIST){
/*        return AddListElements(el1, el2);*/
    }else if(IsArray(el1) || IsArray(el2)){
/*        return AddArrayElements(el1, el2);*/
    }else if(el1->type==DOUBLE || el2->type==DOUBLE){
        if(ToDouble(el2)==0.0){
            EchoError(E_ZeroDiv);
            return NULL;
        }
        return DoubleElement(ToDouble(el1)/ToDouble(el2));
    }else if(el1->type==INT || el1->type==INT){
        if(ToInteger(el2)==0){
            EchoError(E_ZeroDiv);
            return NULL;
        }
        return IntegerElement(ToInteger(el1)/ToInteger(el2));
    }else{
        return NULL;
    }
    return NULL;
}
Element* ModElements(Element* el1, Element* el2)
{
    if(!el1 || !el2) return NULL;
    if(el1->type==STR || el2->type==STR){
        return CatStrings(el1, '%', el2);
    }else if(el1->type==LIST || el2->type==LIST){
/*        return AddListElements(el1, el2);*/
    }else if(IsArray(el1) || IsArray(el2)){
/*        return AddArrayElements(el1, el2);*/
    }else if(el1->type==DOUBLE || el2->type==DOUBLE
             || el1->type==INT || el2->type==INT){
        if(ToInteger(el2)==0){
            EchoError(E_ZeroDiv);
            return NULL;
        }
        return IntegerElement(ToInteger(el1)%ToInteger(el2));
    }else{
        return NULL;
    }
    return NULL;
}
Element* PowElements(Element* el1, Element* el2)
{
    int i;
    if(!el1 || !el2) return NULL;
    if(el1->type==STR || el2->type==STR){
        return CatStrings(el1, '^', el2);
    }else if(el1->type==LIST){
        List* ret;
        List* le;
        if(el2->type!=INT && el2->type!=DOUBLE) return NULL;
        ret=ToList(NULL);
        le=el1->list.next;
        while(le){
            AddToList(ret, PowElements(le->el, el2));
            le=le->next;
        }
        return (Element*)ret;
    }else if(el2->type==LIST){
        List* ret;
        List* le;
        if(el1->type!=INT && el1->type!=DOUBLE) return NULL;
        ret=ToList(NULL);
        le=el2->list.next;
        while(le){
            AddToList(ret, PowElements(el1, le->el));
            le=le->next;
        }
        return (Element*)ret;
    }else if(el1->type==IARRAY){
        double v;
        Element* ret;
        if(el2->type!=INT && el2->type!=DOUBLE) return NULL;
        v=ToDouble(el2);
        ret=DArrayElement(el1->iarray.num, NULL);
        for(i=0; i<el1->iarray.num; i++){
            ret->darray.val[i]
                    =pow((double)(el1->iarray.val[i]), v);
        }
        return ret;
    }else if(el2->type==IARRAY){
        double v;
        Element* ret;
        if(el1->type!=INT && el1->type!=DOUBLE) return NULL;
        v=ToDouble(el1);
        ret=DArrayElement(el2->iarray.num, NULL);
        for(i=0; i<el2->iarray.num; i++){
            ret->darray.val[i]
                    =pow(v, (double)(el2->iarray.val[i]));
        }
        return ret;
    }else if(el1->type==DARRAY){
        double v;
        Element* ret;
        if(el2->type!=INT && el2->type!=DOUBLE) return NULL;
        v=ToDouble(el2);
        ret=DArrayElement(el1->darray.num, NULL);
        for(i=0; i<el1->darray.num; i++){
            ret->darray.val[i]
                    =pow((double)(el1->darray.val[i]), v);
        }
        return ret;
    }else if(el2->type==DARRAY){
        double v;
        Element* ret;
        if(el1->type==INT || el1->type==DOUBLE) return NULL;
        v=ToDouble(el1);
        ret=DArrayElement(el2->darray.num, NULL);
        for(i=0; i<el2->darray.num; i++){
            ret->darray.val[i]
                    =pow(v, (double)(el2->darray.val[i]));
        }
        return ret;
    }else if((el1->type==DOUBLE || el1->type==INT)
             && (el2->type==DOUBLE || el2->type==INT)){
        return DoubleElement(pow(ToDouble(el1), ToDouble(el2)));
    }else{
        return NULL;
    }
}
Element* IncrElement(Element* el, int type)
{
    int v;
    if(type==Incr){
        v=1;
    }else{
        v=-1;
    }
    if(el==NULL){
        return NULL;
    }else if(el->type==INT){
        return IntegerElement(el->integer.val+v);
    }else if(el->type==DOUBLE){
        return DoubleElement(el->real.val+v);
    }else{
        return CopyElement(el);
    }
}
Element* MathElement(Element* el, double(*func)())
{
    int i;
    double v;
    Element* ret;
    if(!el) return NULL;
    if(el->type==INT){
        v=(double)(el->integer.val);
        return DoubleElement(func(v));
    }else if(el->type==DOUBLE){
        return DoubleElement(func(el->real.val));
    }else if(el->type==IARRAY){
        ret=DArrayElement(el->iarray.num, NULL);
        for(i=0; i<el->iarray.num; i++)
                ret->darray.val[i]=
                        func((double)(el->iarray.val[i]));
        return ret;
    }else if(el->type==DARRAY){
        ret=DArrayElement(el->darray.num, NULL);
        for(i=0; i<el->darray.num; i++)
                ret->darray.val[i]=func(el->darray.val[i]);
        return ret;
    }else{
        return NULL;
    }
}
Element* MinVal(Element* el)
{
    int i;
    double min;
    if(!el) return NULL;
    if(el->type==INT || el->type==DOUBLE){
        return DoubleElement(ToDouble(el));
    }else if(el->type==IARRAY){
        min=el->iarray.val[0];
        for(i=1; i<el->iarray.num; i++){
            if(min>el->iarray.val[i]) min=el->iarray.val[i];
        }
        return DoubleElement(min);
    }else if(el->type==DARRAY){
        min=el->darray.val[0];
        for(i=1; i<el->darray.num; i++){
            if(min>el->darray.val[i]) min=el->darray.val[i];
        }
        return DoubleElement(min);
/* case of List */
    }else{
        return NULL;
    }
}
Element* MaxVal(Element* el)
{
    int i;
    double max;
    if(!el) return NULL;
    if(el->type==INT || el->type==DOUBLE){
        return DoubleElement(ToDouble(el));
    }else if(el->type==IARRAY){
        max=el->iarray.val[0];
        for(i=1; i<el->iarray.num; i++){
            if(max<el->iarray.val[i]) max=el->iarray.val[i];
        }
        return DoubleElement(max);
    }else if(el->type==DARRAY){
        max=el->darray.val[0];
        for(i=1; i<el->darray.num; i++){
            if(max<el->darray.val[i]) max=el->darray.val[i];
        }
        return DoubleElement(max);
/* case of List */
    }else{
        return NULL;
    }
}
Element* MinMaxVal(Element* el)
{
    int i;
    double min, max;
    List* ret;
    if(!el) return NULL;
    if(el->type==INT || el->type==DOUBLE){
        ret=ToList(NULL);
        PushToList(ret, DoubleElement(ToDouble(el)));
        PushToList(ret, DoubleElement(ToDouble(el)));
        return (Element*)ret;
    }else if(el->type==IARRAY){
        ret=ToList(NULL);
        min=max=el->iarray.val[0];
        for(i=1; i<el->iarray.num; i++){
            if(max<el->iarray.val[i]){
                max=el->iarray.val[i];
            }else if(min>el->iarray.val[i]){
                min=el->iarray.val[i];
            }
        }
        PushToList(ret, DoubleElement(max));
        PushToList(ret, DoubleElement(min));
        return (Element*)ret;
    }else if(el->type==DARRAY){
        ret=ToList(NULL);
        min=max=el->darray.val[0];
        for(i=1; i<el->darray.num; i++){
            if(max<el->darray.val[i]){
                max=el->darray.val[i];
            }else if(min>el->darray.val[i]){
                min=el->darray.val[i];
            }
        }
        PushToList(ret, DoubleElement(max));
        PushToList(ret, DoubleElement(min));
        return (Element*)ret;
/* case of List */
    }else{
        return NULL;
    }
}

Node* CopyNode(Node* node)
{
    Node* ret;
    if(node==NULL) return NULL;
    ret=AllocateNode();
    ret->type=node->type;
    ret->tmp =TRUE;
    ret->sub =node->sub;
    ret->left =CopyNode(node->left);
    ret->right=CopyNode(node->right);
    ret->el   =CopyElement(node->el);
    return ret;
}
Element* CopyElement(Element* el)
{
    if(!el) return NULL;
    if(el->type==INT){
        return IntegerElement(el->integer.val);
    }else if(el->type==DOUBLE){
        return DoubleElement(el->real.val);
    }else if(el->type==STR){
        return StringElement(el->string.val);
    }else if(el->type==VAR){
        return VariableElement(el->string.val);
    }else if(el->type==COLOR){
        return ColorElement(el->color.pgcl);
    }else if(el->type==FP){
        return FileElement(el->file.fp,
                           el->file.name, el->file.mode);
    }else if(el->type==DATA){
        return DataElement(DuplicateData(el->data.dp));
    }else if(el->type==WINDOW){
        return WindowElement(el->window.wp);
    }else if(el->type==IARRAY){
        return IArrayElement(el->iarray.num, el->iarray.val);
    }else if(el->type==DARRAY){
        return DArrayElement(el->darray.num, el->darray.val);
    }else if(el->type==LIST){
        List* ret;
        List* le;
        ret=ToList(NULL);
        le=el->list.next;
        while(le){
            AddToList(ret, CopyElement(le->el));
            le=le->next;
        }
        return (Element*)ret;
    }else if(el->type==BLOCK){
        Block* blk;
        blk=ToBlock(CopyNode(el->block.node));
        return (Element*)blk;
    }else if(IsNode(el)){
        return (Element*)CopyNode((Node*)el);
    }
    return NULL;
}
int ToBoolean(Element* el)
{
    if(!el){
        return FALSE;
    }else if(el->type==UNDEF){
        return FALSE;
    }else if(el->type==INT){
        if(el->integer.val){
            return TRUE;
        }else{
            return FALSE;
        }
    }else if(el->type==DOUBLE){
        if(el->real.val==0.0){
            return FALSE;
        }else{
            return TRUE;
        }
    }else if(el->type==STR){
        if(el->string.val[0]=='t'
           || el->string.val[0]=='T'
           || el->string.val[0]=='y'
           || el->string.val[0]=='Y'){
            return TRUE;
        }else if(strcmp(el->string.val, "on")==0){
            return TRUE;
        }else{
            return FALSE;
        }
    }else if(el->type==COLOR){
        return TRUE;
    }else if(el->type==FP){
        if(el->file.fp) return TRUE;
        return FALSE;
    }else if(el->type==DATA){
        if(el->data.dp) return TRUE;
        return FALSE;
    }else if(el->type==WINDOW){
        if(el->window.wp) return TRUE;
        return FALSE;
    }else if(el->type==IARRAY){
        if(el->iarray.val) return TRUE;
        return FALSE;
    }else if(el->type==DARRAY){
        if(el->darray.val) return TRUE;
        return FALSE;
    }else if(el->type==LIST){
        return ToBoolean(NthElement(1, (List*)el));
    }else{
        return TRUE;
    }
}
Element* UndefElement()
{
    Undef* el;
    el=(Undef*)malloc(sizeof(Undef));
    el->type=UNDEF;
    el->tmp =TRUE;
    return (Element*)el;
}
int ToInteger(Element* el)
{
    if(!el){
        return 0;
    }else if(el->type==INT){
        return el->integer.val;
    }else if(el->type==DOUBLE){
        return (int)(el->real.val);
    }else if(el->type==STR){
        int ret;
        if(sscanf(el->string.val, "%d", &ret)){
            return ret;
        }else{
            return strlen(el->string.val);
        }
    }else if(el->type==LIST){
        return ToInteger(NthElement(1, (List*)el));
    }else{
        return 0;
    }
}
Element* IntegerElement(int n)
{
    Integer* ret;
    ret=(Integer*)malloc(sizeof(Integer));
    ret->type=INT;
    ret->tmp =TRUE;
    ret->val=n;
    return (Element*)ret;
}
double ToDouble(Element* el)
{
    if(!el){
        return 0.0;
    }else if(el->type==INT){
        return (double)(el->integer.val);
    }else if(el->type==DOUBLE){
        return el->real.val;
    }else if(el->type==STR){
        double ret;
        if(sscanf(el->string.val, "%lf", &ret)){
            return ret;
        }else{
            return 0.0;
        }
    }else if(el->type==LIST){
        return ToDouble(NthElement(1, (List*)el));
    }else{
        return 0.0;
    }
}
Element* DoubleElement(double a)
{
    Double* ret;
    ret=(Double*)malloc(sizeof(Double));
    ret->type=DOUBLE;
    ret->tmp =TRUE;
    ret->val=a;
    return (Element*)ret;
}
char* ToString(Element* el)
{
    char* ret;
    char buf[256];
    if(!el || el->type==UNDEF){
        ret=(char*)malloc(10*sizeof(char));
        sprintf(ret, "(undef)");
        return ret;
    }else if(el->type==INT){
        sprintf(buf, "%d", el->integer.val);
        ret=(char*)malloc((strlen(buf)+1)*sizeof(char));
        strcpy(ret, buf);
        return ret;
    }else if(el->type==DOUBLE){
        sprintf(buf, "%lf", el->real.val);
        ret=(char*)malloc((strlen(buf)+1)*sizeof(char));
        strcpy(ret, buf);
        return ret;
    }else if(el->type==STR || el->type==VAR){
        return el->string.val;
    }else if(el->type==COLOR){
        PGColor pgcl=el->color.pgcl;
        sprintf(buf, "(color: %3d, %3d, %3d)",
                pgcl.r, pgcl.g, pgcl.b);
        ret=(char*)malloc((strlen(buf)+1)*sizeof(char));
        strcpy(ret, buf);
        return ret;
    }else if(el->type==FP){
        sprintf(buf, "(file:%s)", el->file.name);
        ret=(char*)malloc((strlen(buf)+1)*sizeof(char));
        strcpy(ret, buf);
        return ret;
    }else if(el->type==DATA){
        sprintf(buf, "(data:0x%lx)", (long)(el->data.dp));
        ret=(char*)malloc((strlen(buf)+1)*sizeof(char));
        strcpy(ret, buf);
        return ret;
    }else if(el->type==WINDOW){
        sprintf(buf, "(window:0x%lx)", (long)(el->window.wp));
        ret=(char*)malloc((strlen(buf)+1)*sizeof(char));
        strcpy(ret, buf);
        return ret;
    }else if(el->type==IARRAY){
        int i, nbuf=256;
        ret=(char*)malloc(nbuf*sizeof(char));
/*        strcpy(ret, "(");*/
        ret[0]='\0';
        for(i=0; i<el->iarray.num; i++){
            if(nbuf<strlen(ret)+256){
                nbuf+=256;  ret=realloc(ret, nbuf);
            }
            if(0<i) strcat(ret, "\n");
            sprintf(buf, "%d", el->iarray.val[i]);
            strcat(ret, buf);
        }
/*        strcat(ret, ")");*/
        return ret;
    }else if(el->type==DARRAY){
        int i, nbuf=256;
        ret=(char*)malloc(nbuf*sizeof(char));
/*        strcpy(ret, "(");*/
        ret[0]='\0';
        for(i=0; i<el->darray.num; i++){
            if(nbuf<strlen(ret)+256){
                nbuf+=256;  ret=realloc(ret, nbuf);
            }
            if(0<i) strcat(ret, "\n");
            sprintf(buf, "%lf", el->darray.val[i]);
            strcat(ret, buf);
        }
/*        strcat(ret, ")");*/
        return ret;
    }else if(el->type==LIST){
        int i, nbuf=256;
        ret=(char*)malloc(nbuf*sizeof(char));
        strcpy(ret, "(");
        for(i=1; i<=ListLength((List*)el); i++){
            if(1<i) strcat(ret, " ");
            strcat(ret, ToString(NthElement(i, (List*)el)));
            if(nbuf<strlen(ret)+256){
                nbuf+=256;  ret=realloc(ret, nbuf);
            }
        }
        strcat(ret, ")");
        return ret;
    }else if(el->type==BLOCK){
        ret=(char*)malloc(10*sizeof(char));
        sprintf(ret, "(block)");
        return ret;
    }else if(IsNode(el)){
        ret=(char*)malloc(10*sizeof(char));
        sprintf(ret, "(node)");
        return ret;
    }else{
        ret=(char*)malloc(10*sizeof(char));
        sprintf(ret, "(undef)");
        return ret;
    }
}
Element* StringElement(char* str)
{
    String* ret;
    ret=(String*)malloc(sizeof(String));
    ret->type=STR;
    ret->tmp =TRUE;
    ret->val=(char*)malloc((strlen(str)+1)*sizeof(char));
    strcpy(ret->val, str);
    return (Element*)ret;
}
Element* VariableElement(char* str)
{
    String* ret;
    ret=(String*)malloc(sizeof(String));
    ret->type=VAR;
    ret->tmp =TRUE;
    ret->val=(char*)malloc((strlen(str)+1)*sizeof(char));
    strcpy(ret->val, str);
    return (Element*)ret;
}
Element* ColorElement(PGColor pgcl)
{
    Color* ret;
    ret=(Color*)malloc(sizeof(Color));
    ret->type=COLOR;
    ret->tmp =TRUE;
    ret->pgcl=pgcl;
    return (Element*)ret;
}
Element* FileElement(FILE* fp, char* name, char* mode)
{
    FilePointer* ret;
    ret=(FilePointer*)malloc(sizeof(FilePointer));
    ret->type=FP;
    ret->tmp =TRUE;
    ret->fp=fp;
    strcpy(ret->name, name);
    strcpy(ret->mode, mode);
    return (Element*)ret;
}
Element* DataElement(PTRData* dp)
{
    int i, n;
    Data* ret;
    ret=(Data*)malloc(sizeof(Data));
    ret->type=DATA;
    ret->tmp =TRUE;
    if(dp==NULL){
        double x1[2]={0.0, 1.0};
        double x2[2]={0.0, 1.0};
        double x3[2]={0.0, 1.0};
        double  v[2]={0.0, 1.0};
        double* vv[1];
        vv[0]=v;
        dp=CreateSeqData(3, 2, 1, x1, x2, x3, vv);
    }
    ret->dp=dp;
    ret->x1=AllocDArray(dp->nx1, dp->x1);
    if(dp->x1==NULL) dp->x1=ret->x1->darray.val;
    ret->x2=AllocDArray(dp->nx2, dp->x2);
    if(dp->x2==NULL) dp->x2=ret->x2->darray.val;
    ret->x3=AllocDArray(dp->nx3, dp->x3);
    if(dp->x3==NULL) dp->x3=ret->x3->darray.val;
    if(ret->dp->type==GRID){
        n=(dp->nx1)*(dp->nx2)*(dp->nx3);
    }else{
        n=dp->nx1;
    }
    ret->v=ListElement(0, NULL);
    for(i=0; i<dp->nv; i++)
            AddToList((List*)(ret->v), AllocDArray(n, dp->v[i]));
    return (Element*)ret;
}
void UpdateDataElement(Data* data)
{
    int i, n, nv;
    PTRData* dp;
    if(data==NULL || data->type!=DATA) return;
    dp=data->dp;
    if(dp->type==SEQ){
        int n;
        n=dp->nx1;
        if(1<dp->dim && n<dp->nx2) n=dp->nx2;
        if(2<dp->dim && n<dp->nx3) n=dp->nx3;
        dp->nx1=n;
        if(1<dp->dim) dp->nx2=n;
        if(2<dp->dim) dp->nx3=n;
    }
    if(data->x1->darray.num<dp->nx1)
            SetNthValueD(dp->nx1-1, (DArray*)(data->x1), 0.0);
    if(data->x2->darray.num<dp->nx2)
            SetNthValueD(dp->nx2-1, (DArray*)(data->x2), 0.0);
    if(data->x3->darray.num<dp->nx3)
            SetNthValueD(dp->nx3-1, (DArray*)(data->x3), 0.0);
    if(dp->x1!=data->x1->darray.val)
            dp->x1=data->x1->darray.val;
    if(dp->x2!=data->x2->darray.val)
            dp->x2=data->x2->darray.val;
    if(dp->x3!=data->x3->darray.val)
            dp->x3=data->x3->darray.val;
    if(dp->type==GRID){
        n=dp->nx1*dp->nx2*dp->nx3;
    }else if(dp->type==SEQ){
        n=dp->nx1;
    }
    nv=ListLength((List*)(data->v));
    if(nv<dp->nv){
        for(; nv<dp->nv; nv++)
                AddToList((List*)(data->v), DArrayElement(n, NULL));
    }else if(dp->nv<nv){
        dp->v=realloc(dp->v, nv*sizeof(double*));
        for(; dp->nv<nv; (dp->nv)++) dp->v[dp->nv]=NULL;
    }
    for(i=0; i<dp->nv; i++){
        Element* el=NthElement(i+1, (List*)(data->v));
        if(el==NULL || el->type!=DARRAY){
            el=(Element*)ToDArray(el);
            if(el==NULL) el=DArrayElement(n, NULL);
            SetNthElement(i+1, (List*)(data->v), el);
        }
        if(el->darray.num<n)
                SetNthValueD(n-1, (DArray*)(el), 0.0);
        if(dp->v[i]!=el->darray.val)
                dp->v[i]=el->darray.val;
    }
}
Element* WindowElement(PTRWindow* wp)
{
    WindowPointer* ret;
    ret=(WindowPointer*)malloc(sizeof(WindowPointer));
    ret->type=WINDOW;
    ret->tmp =TRUE;
    ret->wp=wp;
    return (Element*)ret;
}
IArray* ToIArray(Element* el)
{
    int i;
    if(el==NULL) return NULL;
    if(el->type==IARRAY){
        return (IArray*)el;
    }else if(el->type==DARRAY){
        IArray* ar;
        ar=(IArray*)IArrayElement(el->darray.num, NULL);
        for(i=0; i<ar->num; i++)
                ar->val[i]=el->darray.val[i];
        return ar;
    }else if(el->type==LIST){
        IArray* ar;
        ar=(IArray*)IArrayElement(ListLength((List*)el), NULL);
        for(i=0; i<ar->num; i++)
                ar->val[i]=ToInteger(NthElement(i+1, (List*)el));
        return ar;
    }else{
        return NULL;
    }
}
Element* IArrayElement(int n, int* val)
{
    int i;
    IArray* ret;
    ret=(IArray*)malloc(sizeof(IArray));
    ret->type=IARRAY;
    ret->tmp =TRUE;
    ret->num =n;
    ret->val=(int*)malloc(n*sizeof(int));
    if(val){
        memcpy(ret->val, val, n*sizeof(int));
    }else{
        for(i=0; i<n; i++) ret->val[i]=0;
    }
    return (Element*)ret;
}
DArray* ToDArray(Element* el)
{
    int i;
    if(el==NULL) return NULL;
    if(el->type==DARRAY){
        return (DArray*)el;
    }else if(el->type==IARRAY){
        DArray* ar;
        ar=(DArray*)DArrayElement(el->iarray.num, NULL);
        for(i=0; i<ar->num; i++)
                ar->val[i]=el->iarray.val[i];
        return ar;
    }else if(el->type==LIST){
        DArray* ar;
        ar=(DArray*)DArrayElement(ListLength((List*)el), NULL);
        for(i=0; i<ar->num; i++)
                ar->val[i]=ToDouble(NthElement(i+1, (List*)el));
        return ar;
    }else{
        return NULL;
    }
}
Element* DArrayElement(int n, double* val)
{
    int i;
    DArray* ret;
    ret=(DArray*)malloc(sizeof(DArray));
    ret->type=DARRAY;
    ret->tmp =TRUE;
    ret->num =n;
    ret->val=(double*)malloc(n*sizeof(double));
    if(val){
        memcpy(ret->val, val, n*sizeof(double));
    }else{
        for(i=0; i<n; i++) ret->val[i]=0.0;
    }
    return (Element*)ret;
}
Element* AllocDArray(int n, double* val)
{
    int i;
    DArray* ret;
    ret=(DArray*)malloc(sizeof(DArray));
    ret->type=DARRAY;
    ret->tmp =TRUE;
    ret->num =n;
    if(val){
        ret->val=val;
    }else{
        ret->val=(double*)malloc(n*sizeof(double));
        for(i=0; i<n; i++) ret->val[i]=0.0;
    }
    return (Element*)ret;
}
List* ToList(Element* el)
{
    List* ret;
    ret=(List*)malloc(sizeof(List));
    ret->type=LIST;
    ret->tmp =TRUE;
    ret->el  =NULL;
    ret->next=NULL;
    if(!el){
        return ret;
    }else if(el->type!=LIST){
        List* le;
        le=(List*)malloc(sizeof(List));
        le->type=LIST;
        le->tmp =TRUE;
        le->el=el;
        le->next=ret->next;
        ret->next=le;
    }else{
        free(ret);
        return (List*)el;
    }
    return ret;
}
Element* ListElement(int n, Element* el[])
{
    int i;
    List* list;
    list=(List*)malloc(sizeof(List));
    list->type=LIST;
    list->tmp =TRUE;
    list->el  =NULL;
    list->next=NULL;
    for(i=n-1; 0<=i; i--) PushToList(list, CopyElement(el[i]));
    return (Element*)list;
}
Block* ToBlock(Node* node)
{
    Block* blk;
    blk=(Block*)malloc(sizeof(Block));
    blk->type=BLOCK;
    blk->tmp =TRUE;
    blk->node=node;
    blk->vl=AllocateVariable("");
    return blk;
}
Element* BlockElement(Node* node)
{
    Block* blk;
    blk=(Block*)malloc(sizeof(Block));
    blk->type=BLOCK;
    blk->tmp =TRUE;
    blk->node=node;
    blk->vl=AllocateVariable("");
    return (Element*)blk;
}
Node* AllocateNode(){
    Node* ret;
    ret=(Node*)malloc(sizeof(Node));
    ret->type =UNDEF;
    ret->tmp  =TRUE;
    ret->left =NULL;
    ret->right=NULL;
    ret->el   =NULL;
    return ret;
}

Element* ToElement(char* str)
{
    int i, j;
    StripSpaces(&str);
/* NULL */
    if(str[0]=='\0') return NULL;
    if(str[0]=='.' && !isdigit(str[1])) return StringElement(str);
/* Integer */
    i=0;
    if(str[i]=='+' || str[i]=='-') i++;
    for(; str[i]!='\0'; i++){
        if(!isdigit(str[i])) break;
    }
    if(str[i]=='\0') return IntegerElement(atoi(str));
/* Double */
    if(str[i]=='.'){
        for(i++; str[i]!='\0'; i++){
            if(!isdigit(str[i])) break;
        }
        if(str[i]=='\0') return DoubleElement(atof(str));
    }
    switch(str[i]){
      case 'e':
      case 'E':
      case 'd':
      case 'D':
        j=i;
        if(str[i+1]=='+' || str[i+1]=='-') i++;
        for(i++; str[i]!='\0'; i++){
            if(!isdigit(str[i])) break;
        }
        if(str[i]=='\0'){
            str[j]='e';
            return DoubleElement(atof(str));
        }
    }
/* Variable */
    if(str[0]=='$'){
        Element* el;
        for(i=0; str[i]!='\0'; i++){
            if(!IsVarWord(str[i])) return StringElement(str);
        }
        el=StringElement(str);
        el->type=VAR;
        return el;
    }
/* String */
    return StringElement(str);
}

Element* StructMember(Element* st, char* str)
{
    char msg[256];
    if(st==NULL) return NULL;
    if(strcmp(str, "type")==0)
            return StringElement(TypeToName(st->type));
    if(st->type==STR){
        char* buf;
        Element* ret;
        buf=(char*)malloc((strlen(st->string.val)+strlen(str)+3)
                          *sizeof(char));
        sprintf(buf, "%s.%s", st->string.val, str);
        ret=StringElement(buf);
        free(buf);
        return ret;
    }else if(st->type==FP){
        if(strcmp(str, "name")==0){
            return StringElement(st->file.name);
        }else if(strcmp(str, "mode")==0){
            return StringElement(st->file.mode);
        }
    }else if(st->type==DATA){
        PTRData* dp;
        dp=st->data.dp;
        if(dp==NULL) return NULL;
        if(strcmp(str, "dtype")==0){
            return StringElement(DataTypeToName(dp->type));
        }else if(strcmp(str, "dim")==0){
            return IntegerElement(dp->dim);
        }else if(strcmp(str, "nx1")==0
                 || strcmp(str, "nx")==0){
            return IntegerElement(dp->nx1);
        }else if(strcmp(str, "nx2")==0
                 || strcmp(str, "ny")==0){
            return IntegerElement(dp->nx2);
        }else if(strcmp(str, "nx3")==0
                 || strcmp(str, "nz")==0){
            return IntegerElement(dp->nx3);
        }else if(strcmp(str, "nv")==0){
            return IntegerElement(dp->nv);
        }else if(strcmp(str, "x1")==0
                 || strcmp(str, "x")==0){
            SetTmpVal(st->data.x1, FALSE);
            return st->data.x1;
        }else if(strcmp(str, "x2")==0
                 || strcmp(str, "y")==0){
            SetTmpVal(st->data.x2, FALSE);
            return st->data.x2;
        }else if(strcmp(str, "x3")==0
                 || strcmp(str, "z")==0){
            SetTmpVal(st->data.x3, FALSE);
            return st->data.x3;
        }else if(strcmp(str, "v")==0){
            SetTmpVal(st->data.v, FALSE);
            return st->data.v;
        }
    }else if(st->type==WINDOW){
        PTRWindow* wp;
        wp=st->window.wp;
        if(wp==NULL) return NULL;
        if(strcmp(str, "parent")==0){
            return WindowElement(wp->parent);
        }else if(strcmp(str, "wdx")==0){
            return IntegerElement(wp->pgw->wdx);
        }else if(strcmp(str, "wdy")==0){
            return IntegerElement(wp->pgw->wdy);
        }else if(strcmp(str, "mag")==0){
            return DoubleElement(wp->pgw->mag);
        }else if(strcmp(str, "orgx")==0){
            return DoubleElement(wp->pgw->orgx);
        }else if(strcmp(str, "orgy")==0){
            return DoubleElement(wp->pgw->orgy);
        }else if(strcmp(str, "coord")==0){
            return StringElement(CoordToName(wp->coord));
        }else if(strcmp(str, "ax1")==0){
            return StringElement(AxisToName(wp->ax1));
        }else if(strcmp(str, "ax2")==0){
            return StringElement(AxisToName(wp->ax2));
        }else if(strcmp(str, "ax3")==0){
            return StringElement(AxisToName(wp->ax3));
        }else if(strcmp(str, "exact")==0){
            return IntegerElement(wp->exact);
        }else if(strcmp(str, "x1min")==0){
            return DoubleElement(X1Min(wp));
        }else if(strcmp(str, "x1max")==0){
            return DoubleElement(X1Max(wp));
        }else if(strcmp(str, "x2min")==0){
            return DoubleElement(X2Min(wp));
        }else if(strcmp(str, "x2max")==0){
            return DoubleElement(X2Max(wp));
        }else if(strcmp(str, "x3min")==0){
            return DoubleElement(X3Min(wp));
        }else if(strcmp(str, "x3max")==0){
            return DoubleElement(X3Max(wp));
        }else if(strcmp(str, "vmin")==0){
            return DoubleElement(VMin(wp));
        }else if(strcmp(str, "vmax")==0){
            return DoubleElement(VMax(wp));
        }else if(strcmp(str, "xmin")==0){
            return DoubleElement(XMin(wp));
        }else if(strcmp(str, "xmax")==0){
            return DoubleElement(XMax(wp));
        }else if(strcmp(str, "ymin")==0){
            return DoubleElement(YMin(wp));
        }else if(strcmp(str, "ymax")==0){
            return DoubleElement(YMax(wp));
        }else if(strcmp(str, "zmin")==0){
            return DoubleElement(ZMin(wp));
        }else if(strcmp(str, "zmax")==0){
            return DoubleElement(ZMax(wp));
        }else if(strcmp(str, "phi")==0){
            return DoubleElement(wp->pgw->phi);
        }else if(strcmp(str, "theta")==0){
            return DoubleElement(wp->pgw->theta);
        }else if(strcmp(str, "y_x")==0){
            return DoubleElement(wp->pgw->x2_x1);
        }else if(strcmp(str, "z_x")==0){
            return DoubleElement(wp->pgw->x3_x1);
        }
    }else if(st->type==IARRAY){
        if(strcmp(str, "length")==0
           || strcmp(str, "l")==0){
            return IntegerElement(st->iarray.num);
        }
    }else if(st->type==DARRAY){
        if(strcmp(str, "length")==0
           || strcmp(str, "l")==0){
            return IntegerElement(st->darray.num);
        }
    }else if(st->type==LIST){
        if(strcmp(str, "length")==0
           || strcmp(str, "l")==0){
            return IntegerElement(ListLength((List*)st));
        }
    }
    sprintf(msg, E_NoMember, str);
    ExecError(msg);
    return NULL;
}
void SetStructMember(Element* st, char* str, Element* el)
{
    char msg[256];
    if(st==NULL) return;
    if(strcmp(str, "type")==0){
        sprintf(msg, E_MemberSubst, "type");
        ExecError(msg);
        return;
    }
    if(st->type==FP){
        if(strcmp(str, "name")==0
           || strcmp(str, "mode")==0){
            sprintf(msg, E_MemberSubst, str);
            ExecError(msg);
            return;
        }
    }else if(st->type==DATA){
        PTRData* dp;
        dp=st->data.dp;
        if(dp==NULL) return;
        if(strcmp(str, "dtype")==0){
            int type;
            type=NameToDataType(ToString(el));
            if(type==0){
                sprintf(msg, E_NoDataType, ToString(el));
                ExecError(msg);
                return;
            }
            dp->type=type;
            UpdateDataElement((Data*)st);
            return;
        }else if(strcmp(str, "dim")==0){
            int dim;
            dim=ToInteger(el);
            if(dim<1 || 3<dim){
                return;
            }
            dp->dim=dim;
            return;
        }else if(strcmp(str, "nx1")==0
                 || strcmp(str, "nx")==0){
            int n;
            n=ToInteger(el);
            if(n<1){
                return;
            }
            dp->nx1=n;
            UpdateDataElement((Data*)st);
            return;
        }else if(strcmp(str, "nx2")==0
                 || strcmp(str, "ny")==0){
            int n;
            n=ToInteger(el);
            if(n<1){
                return;
            }
            dp->nx2=n;
            UpdateDataElement((Data*)st);
            return;
        }else if(strcmp(str, "nx3")==0
                 || strcmp(str, "nz")==0){
            int n;
            n=ToInteger(el);
            if(n<1){
                return;
            }
            dp->nx3=n;
            UpdateDataElement((Data*)st);
            return;
        }else if(strcmp(str, "nv")==0){
            int n;
            n=ToInteger(el);
            if(n<1){
                return;
            }
            dp->nv=n;
            UpdateDataElement((Data*)st);
            return;
        }else if(strcmp(str, "x1")==0
                 || strcmp(str, "x")==0){
            DArray* ar;
            ar=ToDArray(el);
            if(ar==NULL){
                FreeElement(el);
                return;
            }else if((Element*)ar!=el){
                FreeElement(el);
            }
            FreeElement(st->data.x1);
            st->data.x1=(Element*)ar;
            SetTmpVal(st->data.x1, FALSE);
            UpdateDataElement((Data*)st);
            return;
        }else if(strcmp(str, "x2")==0
                 || strcmp(str, "y")==0){
            DArray* ar;
            ar=ToDArray(el);
            if(ar==NULL){
                FreeElement(el);
                return;
            }else if((Element*)ar!=el){
                FreeElement(el);
            }
            FreeElement(st->data.x2);
            st->data.x2=(Element*)ar;
            SetTmpVal(st->data.x2, FALSE);
            UpdateDataElement((Data*)st);
            return;
        }else if(strcmp(str, "x3")==0
                 || strcmp(str, "z")==0){
            DArray* ar;
            ar=ToDArray(el);
            if(ar==NULL){
                FreeElement(el);
                return;
            }else if((Element*)ar!=el){
                FreeElement(el);
            }
            FreeElement(st->data.x3);
            st->data.x3=(Element*)ar;
            SetTmpVal(st->data.x3, FALSE);
            UpdateDataElement((Data*)st);
            return;
        }else if(strcmp(str, "v")==0){
            int i, n;
            DArray* ar;
            Element* el1;
            List* list;
            if(el==NULL || el->type!=LIST) return;
            n=ListLength((List*)el);
            list=ToList(NULL);
            for(i=0; i<n; i++){
                el1=NthElement(i+1, (List*)el);
                ar=ToDArray(el1);
                if(ar==NULL){
                    FreeElement(el);  FreeList(list);
                    return;
                }else if((Element*)ar==el1){
                    ar=(DArray*)CopyElement(el);
                }
                AddToList(list, (Element*)ar);
            }
            FreeElement(st->data.v);
            st->data.v=(Element*)list;
            SetTmpVal(st->data.v, FALSE);
            UpdateDataElement((Data*)st);
            FreeElement(el);
            return;
        }
    }else if(st->type==WINDOW){
        PTRWindow* wp;
        wp=st->window.wp;
        if(wp==NULL) return;
        if(strcmp(str, "parent")==0){
            sprintf(msg, E_MemberSubst, str);
            ExecError(msg);
            return;
        }else if(strcmp(str, "mag")==0){
            SetMagnify(wp, ToDouble(el));
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "orgx")==0){
            SetOriginX(wp, ToDouble(el));
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "orgy")==0){
            SetOriginY(wp, ToDouble(el));
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "coord")==0){
            int coord;
            coord=NameToCoord(ToString(el));
            if(coord==0){
                sprintf(msg, E_NoSuchCoord, ToString(el));
                ExecError(msg);
                return;
            }
            SetCoord(wp, coord);
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "ax1")==0 || strcmp(str, "ax")==0){
            int ax;
            ax=NameToAxis(ToString(el));
            if(ax<0 || 4<ax){
                sprintf(msg, E_NoSuchAxis, ToString(el));
                ExecError(msg);
                return;
            }
            SetX1Axis(wp, ax);
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "ax2")==0 || strcmp(str, "ay")==0){
            int ax;
            ax=NameToAxis(ToString(el));
            if(ax<0 || 4<ax){
                sprintf(msg, E_NoSuchAxis, ToString(el));
                ExecError(msg);
                return;
            }
            SetX2Axis(wp, ax);
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "ax3")==0 || strcmp(str, "az")==0){
            int ax;
            ax=NameToAxis(ToString(el));
            if(ax<0 || 4<ax){
                sprintf(msg, E_NoSuchAxis, ToString(el));
                ExecError(msg);
                return;
            }
            SetX3Axis(wp, ax);
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "exact")==0){
            ExactRatio(wp, ToBoolean(el));
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "x1min")==0){
            SetX1Min(wp, ToDouble(el));
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "x1max")==0){
            SetX1Max(wp, ToDouble(el));
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "x2min")==0){
            SetX2Min(wp, ToDouble(el));
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "x2max")==0){
            SetX2Max(wp, ToDouble(el));
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "x3min")==0){
            SetX3Min(wp, ToDouble(el));
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "x3max")==0){
            SetX3Max(wp, ToDouble(el));
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "vmin")==0){
            SetVMin(wp, ToDouble(el));
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "vmax")==0){
            SetVMax(wp, ToDouble(el));
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "xmin")==0){
            SetXMin(wp, ToDouble(el));
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "xmax")==0){
            SetXMax(wp, ToDouble(el));
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "ymin")==0){
            SetYMin(wp, ToDouble(el));
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "ymax")==0){
            SetYMax(wp, ToDouble(el));
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "zmin")==0){
            SetZMin(wp, ToDouble(el));
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "zmax")==0){
            SetZMax(wp, ToDouble(el));
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "phi")==0){
            SetPhi(wp, ToDouble(el));
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "theta")==0){
            SetTheta(wp, ToDouble(el));
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "y_x")==0){
            SetY_X(wp, ToDouble(el));
            ShowWindow(wp);
            return;
        }else if(strcmp(str, "z_x")==0){
            SetZ_X(wp, ToDouble(el));
            ShowWindow(wp);
            return;
        }
    }else if(st->type==IARRAY){
        if(strcmp(str, "length")==0
           || strcmp(str, "l")==0){
            int n;
            n=ToInteger(el);
            if(st->iarray.num<n)
                    SetNthValueI(n-1, (IArray*)st, 0);
            st->iarray.num=n;
        }
    }else if(st->type==DARRAY){
        if(strcmp(str, "length")==0
           || strcmp(str, "l")==0){
            int n;
            n=ToInteger(el);
            if(st->darray.num<n)
                    SetNthValueD(n-1, (DArray*)st, 0.0);
            st->darray.num=n;
        }
    }else if(st->type==LIST){
        if(strcmp(str, "length")==0
           || strcmp(str, "l")==0){
            return;
        }
    }
    sprintf(msg, E_NoMember, str);
    ExecError(msg);
    return;
}


/* list operation */
Element* SearchElement(List* list, Element* el)
{
    Element* ret;
    List* le;
    le=list->next;
    while(le){
        if(!(le->el)){
        }else if(le->el->type==LIST){
            if((ret=SearchElement((List*)(le->el), el)))
                    return ret;
        }else if(IsSameElement(le->el, el)){
            return le->el;
        }
        le=le->next;
    }
    return NULL;
}
Element* PopFromList(List* list)
{
    List* le;
    Element* ret;
    le=list->next;
    if(!le) return NULL;
    list->next=le->next;
    ret=le->el;
    free(le);
    return ret;
}
void AddToList(List* list, Element* el)
{
    List* le;
    List* nle;
    le=list;
    while(le->next) le=le->next;
    nle=(List*)malloc(sizeof(List));
    nle->el=el;
    nle->next=le->next;
    le->next=nle;
}
void PushToList(List* list, Element* el)
{
    List* le;
    le=(List*)malloc(sizeof(List));
    le->next=list->next;
    le->el=el;
    list->next=le;
}
List* CatList(List* list1, List* list2)
{
    List* le;
    if(list2==NULL) return list1;
    if(list1==NULL) return list2;
    le=list1->next;
    while(le->next) le=le->next;
    while(list2->next){
        le->next=ToList(NULL);
        le=le->next;
        le->el=PopFromList(list2);
    }
    return list1;
}
int ListLength(List* list)
{
    int i=0;
    List* le;
    if(!list) return 0;
    le=list->next;
    while(le){
        i++;  le=le->next;
    }
    return i;
}
Element* NthElement(int n, List* list)
{
    int i=0;
    List* le;
    if(!list) return NULL;
    if(n<1) return NULL;
    if(list->type!=LIST){
        if(n==1) return (Element*)list;
        return NULL;
    }
    le=list->next;
    while(le){
        i++;
        if(i==n) return le->el;
        le=le->next;
    }
    return (Element*)NULL;
}
void SetNthElement(int n, List* list, Element* el)
{
    int i;
    List* le;
    if(!list) return;
    if(n<1) return;
    le=list;
    for(i=0; i<n; i++){
        if(le->next==NULL) le->next=ToList(NULL);
        le=le->next;
    }
    FreeElement(le->el);
    le->el=el;
}
Element** PrepareNthElement(int n, List* list)
{
    int i;
    List* le;
    if(!list) return NULL;
    if(n<1) return NULL;
    le=list;
    for(i=0; i<n; i++){
        if(le->next==NULL){
            le->next=ToList(NULL);
            le->next->tmp=list->tmp;
        }
        le=le->next;
    }
    return &(le->el);
}
Element* NthValueI(int n, IArray* array)
{
    if(array==NULL) return NULL;
    if(n<0 || array->num-1<n) return NULL;
    return IntegerElement(array->val[n]);
}
Element* NthValueD(int n, DArray* array)
{
    if(array==NULL) return NULL;
    if(n<0 || array->num-1<n) return NULL;
    return DoubleElement(array->val[n]);
}
void SetNthValueI(int n, IArray* array, int val)
{
    int i;
    if(array->num-1<n){
        array->val=realloc(array->val, (n+1)*sizeof(int));
        for(i=array->num; i<n; i++) array->val[i]=0;
        array->num=n+1;
    }
    array->val[n]=val;
}
void SetNthValueD(int n, DArray* array, double val)
{
    int i;
    if(array->num-1<n){
        array->val=realloc(array->val, (n+1)*sizeof(double));
        for(i=array->num; i<n; i++) array->val[i]=0.0;
        array->num=n+1;
    }
    array->val[n]=val;
}

/* associated array operation */
List* SearchKey(List* list, Element* key)
{
    List* le;
    le=list->next;
    while(le){
        if(IsSameElement(le->el, key)) return le;
        le=le->next;
        if(!le) return NULL;
        le=le->next;
    }
    return NULL;
}
Element* KeyToValue(List* list, Element* key)
{
    List* le;
    le=SearchKey(list, key);
    if(le && le->next) return le->next->el;
    return NULL;
}
void AddToAsArray(List* list, Element* key, Element* val)
{
    List* le;
    le=SearchKey(list, key);
    if(le==NULL){
        PushToList(list, val);
        PushToList(list, key);
    }else if(le->next){
        FreeElement(le->next->el);
        le->next->el=val;
    }else{
        AddToList(list, val);
    }
}
Element** PrepareKeyToValue(List* list, Element* key)
{
    List* le;
    le=SearchKey(list, key);
    if(le==NULL){
        AddToAsArray(list, CopyElement(key), NULL);
        le=SearchKey(list, key);
    }else if(le->next==NULL){
        le->next=ToList(NULL);
        le->next->tmp=list->tmp;
    }
    return &(le->next->el);
}

/* variables */
VariableList* AllocateVariable(char* name)
{
    VariableList* vl;
    vl=(VariableList*)malloc(sizeof(VariableList));
    strcpy(vl->name, name);
    vl->global=FALSE;
    vl->el=NULL;
    vl->next=NULL;
    return vl;
}
Element* SearchVarElement(VariableList* vlist, Element* el)
{
    Element* ret;
    VariableList* vl;
    vl=vlist->next;
    while(vl){
        if(!(vl->el)){
        }else if(vl->el->type==LIST){
            if((ret=SearchElement((List*)(vl->el), el)))
                    return ret;
        }else if(IsSameElement(vl->el, el)){
            return vl->el;
        }
        vl=vl->next;
    }
    return NULL;
}

Element* VarNameToElement(char* name)
{
    VariableList* vl;
    vl=crvl->next;
    while(vl && (strcmp(vl->name, name)!=0)){
        vl=vl->next;
    }
    if(vl==NULL) return NULL;
    if(crvl==&global || !(vl->global)) return vl->el;
    vl=global.next;
    while(vl && (strcmp(vl->name, name)!=0)){
        vl=vl->next;
    }
    if(vl) return vl->el;
    return NULL;
}
Element** PrepareVariable(char* name)
{
    VariableList* vl;
    vl=crvl;
    while(strcmp(vl->name, name)!=0){
        if(vl->next==NULL) vl->next=AllocateVariable(name);
        vl=vl->next;
    }
    if(crvl==&global || !(vl->global)) return &(vl->el);
    vl=&global;
    while(strcmp(vl->name, name)!=0){
        if(vl->next==NULL) vl->next=AllocateVariable(name);
        vl=vl->next;
    }
    return &(vl->el);
}
int VariableExists(char* name, Element** el)
{
    VariableList* vl;
    vl=crvl->next;
    while(vl && (strcmp(vl->name, name)!=0)) vl=vl->next;
    if(vl==NULL){
        *el=NULL;
        return FALSE;
    }else if(crvl==&global || !(vl->global)){
        *el=vl->el;
        return TRUE;
    }
    vl=global.next;
    while(vl && (strcmp(vl->name, name)!=0)) vl=vl->next;
    if(vl){
        *el=vl->el;
        return TRUE;
    }
    *el=NULL;
    return FALSE;
}
void SetVariable(char* name, Element* el)
{
    Element** ep;
    ep=PrepareVariable(name);
    FreeElement(*ep);
    SetTmpVal(el, FALSE);
    *ep=el;
}
void PushVariable(char* name, Element* el)
{
    Element** ep;
    ep=PrepareVariable(name);
    if(!(*ep) || (*ep)->type!=LIST){
        List* list;
        list=(List*)malloc(sizeof(List));
        list=ToList(*ep);
        *ep=(Element*)list;
    }
    SetTmpVal(el, FALSE);
    PushToList((List*)(*ep), el);
}
int VariableLength(char* name)
{
    Element* el;
    el=VarNameToElement(name);
    if(el){
        if(el->type==LIST){
            return ListLength((List*)el);
        }else{
            return 1;
        }
    }else{
        return 0;
    }
}

Element* GetNthElement(int n, char* name)
{
    Element* el;
    el=VarNameToElement(name);
    if(!el || n<1){
        return NULL;
    }else if(el->type==LIST){
        return NthElement(n, (List*)el);
    }else if(n==1){
        return el;
    }
    return NULL;
}

Element* GetAsArrayValue(char* name, Element* key)
{
    Element* el;
    el=VarNameToElement(name);
    if(!el || el->type!=LIST) return NULL;
    return KeyToValue((List*)el, key);
}
void AddToAsArrayVar(char* name, Element* key, Element* val)
{
    Element* el;
    el=VarNameToElement(name);
    if(!el || el->type!=LIST){
        el=ListElement(0, NULL);
        SetVariable(name, el);
    }
    SetTmpVal(key, FALSE);
    SetTmpVal(val, FALSE);
    AddToAsArray((List*)el, key, val);
}
VariableList* GlobalVarList(){
    return &global;
}
void SetCurrentVarList(VariableList* vl){
    crvl=vl;
}
void Globalize(char* name)
{
    VariableList* vl;
    if(crvl==&global) return;
    vl=crvl->next;
    while(vl && (strcmp(vl->name, name)!=0)) vl=vl->next;
    if(vl==NULL){
        vl=AllocateVariable(name);
        vl->next=crvl->next;
        crvl->next=vl;
        vl->global=TRUE;
    }else if(!(vl->global)){
        FreeElement(vl->el);  vl->el=NULL;
        vl->global=TRUE;
    }
}

/* default variables */

void DefaultVariable(char* name){
    strcpy(dfvname, name);
}
void ClearDefaultVariable(){
    dfvname[0]='\0';
}
void SetDefaultVariable(Element* el)
{
    if(dfvname[0]=='\0') return;
    SetVariable(dfvname, el);
    ClearDefaultVariable();
}
